; This is a working source file for Zeus, the port of Neil Mottershead's Zeus assembler from the Nascom-2 to the ZX Spectrum.
;
; The original source is long-gone so this is reconstruced using Diana and barely-remembered routine names.
;
; Zeus is (c) Neil Mottershead and Simon Brattel (Crem), 1983 .. 2020, for what it's worth, though everyone and their dog steals
; ideas from it and always has. Some software houses just changed the name and used it as their in-house assembler, FFS.
;
; Zeus was incredibly fast and used very little memory for source-code, mainly because it was the first assembler to use tokenising.
;
; Zeus was so fast that in one influential review a bloody stupid magazine reviewer didn't realise it had assembled their test
; source (as it took a small fraction of a second, less than a cursor-blink) and so gave the "fastest assembler" rating to something
; well over sixty times slower than Zeus, the unspeakable fecking moron... how we cursed him ;)
;
; If you improve this disassembly, or spot errors in it, please contact me (Simon Brattel) if I'm still alive and I'll correct it.
; There are still quite a few labels that haven't been humanised, and some of the "token" ones are questionable.
;
; This source assembles and emulates using the Zeus Assembler which can be found here:
; https://www.desdes.com/products/oldfiles/

                        emulate_spectrum "48K"          ; Zeus 1 was written for the 48K Spectrum, so emulate that

; Zeus makes only one ROM call. We hated that unspeakable joke of a ROM, but needed to support printing.

ROM_ChanOpen            equ $1601                       ;

VDU_Base                equ $4000                       ;
lPixels                 equ $1800                       ;
ATTR_Base               equ VDU_Base+lPixels            ;
lAttrs                  equ $300                        ;

; System variables used by Zeus (48K original, these will have been investigated by us as there was little or no doc back then)
; Mostly for key input, we didn't scan the keyboard ourselves to keep the size of Zeus down.

ROM_Unk                 equ $5C05                       ;
ROM_LastK               equ $5C08                       ;
ROM_Mode                equ $5C41                       ;
ROM_Flags2              equ $5C6A                       ;
ROM_CharsPtr            equ $5C36                       ;

; Zeus's internal printing control codes

cDELRIGHT               equ $04                         ;
cINSERT                 equ $05                         ;
cTAB                    equ $06                         ;
cCLEARLINE              equ $07                         ;
cLEFT                   equ $08                         ;
cRIGHT                  equ $09                         ;
cLF                     equ $0A                         ;
cUP                     equ $0B                         ;
cDELETE                 equ $0C                         ;
cCR                     equ $0D                         ;
cCLS                    equ $0F                         ;

; Code starts here
; Assembler stuff to start with, written almost entirely by Neil...

                        org $E000                       ;

Zeus_PC                 equ *                           ;

Start                   push hl                         ;
                        push de                         ;
                        push bc                         ;
                        push af                         ;
                        ex af,af'                       ;
                        exx                             ;
                        push hl                         ;
                        push de                         ;
                        push bc                         ;
                        push af                         ;
                        ex af,af'                       ;
                        exx                             ;
                        ld (EntryStackBase),sp          ;

                        ; Saved registers on stack, stored SP

                        ; Setup attributes (white/blue)
                        ld a,$0F                        ;
                        ld (CLS_cAttr),A                ;

                        ; Setup border colour (blue)
                        ld a,$01                        ;
                        ld (BdrCol),A                   ;

                        ; Open channel 3 (some ROM crap)
                        ld a,3                          ;
                        call ROM_ChanOpen               ;

                        ; Disable printing
                        xor a                           ;
                        ld (PrinterFlag),A              ;

                        ; And something else
                        ld (ROM_Mode),A                 ;

                        ; Gawd knows
                        ld a,$F7                        ;
                        ld hl,ROM_Flags2                ;
                        and (hl)                        ;
                        ld (hl),a                       ;

                        ; Announce us.
                        call PrintStringZ               ;
                        db cCLS                         ;
                        db "   ZEUS Z80 ASSEMBLER"      ;
                        db cCR,$7F                      ;
                        db " 1983 Crystal Computing",cCR,cCR,0;

                        ; These NOPs are here because some bastard clumsily removed our names :(
                        ; We knew they probably would, so we hid them in the code.

                        nop                             ;
                        nop                             ;
                        nop                             ;
                        nop                             ;
                        nop                             ;
                        nop                             ;
                        nop                             ;

                        ; Setup the default tab positiopns

                        ld hl,TabStopDefn               ;
                        ld de,TabStopBuffer             ;
                        ld bc,33                        ;
                        ldir                            ;

                        ld a,$FF                        ;
                        ld (NoSourceFlag),A             ;

                        ; Check if this is a restart, do we have the magic number "42" in memory?

                        ld hl,(MagicNumber)             ;
                        ld a,l                          ;
                        cp '4'                          ;
                        jr nz MainCold                  ;

                        ld a,h                          ;
                        cp '2'                          ;
                        jr z MainWarm                   ;

                        ; Start from cold

MainCold                ld hl,'24'                      ; Magic number is "42", but reverse in the load
                        ld (MagicNumber),hl             ;
                        ld hl,$8000                     ;
                        ld (SourceStartAddr),hl         ;

                        ; Warm start

MainWarm                ld de,$FFFF                     ;
                        call FindLineNum_DE             ;
                        dec hl                          ;
                        ld (SourceEndAddr),hl           ;

                        ld hl,LabelBuffer               ;
                        ld (SymbolTableBase),hl         ;

                        ld hl,AssShowError              ;
                        ld (ExecuteAddr),hl             ;

                        call PrintStringZ               ;
                        db "ZEUS    ",cCR,cCR,0         ;

MainLp                  xor a                           ;
                        ld (LastCmdIndexOrAssFlags),A   ;

AssEntryLp              ld a,(LastCmdIndexOrAssFlags)   ;
                        cp 4                            ; Look at the cmd index to see if it was an "I", if so we're adding line numbers, I think
                        jr nz MainGetEntryLine          ;

                        ; Add the next auto-index line number

                        ld hl,(AssParam1)               ;
                        ld de,(AssParam2)               ;
                        add hl,de                       ;
                        jr c AssShowError               ;

MainInsertLN            call PrintDecimal               ;

MainGetEntryLine        ld sp,(EntryStackBase)          ;
                        call InputLine                  ;
                        call Skip                       ;
                        jr z MainLp                     ;

                        call IfLetterUpperOnly_NC       ;
                        jr c AssNotLetter               ;

                        call AssDealWithCmd             ;
                        jr nc AssEntryLp                ;

                        jr AssShowError                 ;

AssNotLetter            call IfNumber_NC                ;
                        jr c AssShowError               ;

                        call AssHaveLineNumber          ;
                        jr nc AssEntryLp                ;

AssShowError            call PrintStringZ               ;
                        db "Error",cCR,0                ;

                        jr MainLp                       ;

AssDealWithCmd          ld hl,AssCmdTable               ;
                        ld b,$00                        ;
                        call Tokenize                   ;
                        ret c                           ;

                        push de                         ;
                        ld a,b                          ;
                        ld (LastCmdIndexOrAssFlags),A   ;
                        ld hl,AddCmdOffsets             ;
                        jr DriveTable                   ;

DT_Lp                   add a,l                         ;
                        ld l,a                          ;
                        jr nc DriveTable                ;

                        inc h                           ;
DriveTable              ld a,(hl)                       ;
                        inc hl                          ;
                        inc a                           ;
                        add a,a                         ;
                        djnz DT_Lp                      ;

                        sub $02                         ;
                        jr z DT_GetNextWord             ;

                        ld c,a                          ;
                        ld de,AssParam1                 ;
                        ldir                            ;

DT_GetNextWord          ld a,(hl)                       ;
                        inc hl                          ;
                        ld h,(hl)                       ;
                        ld l,a                          ;

                        pop de                          ;
                        push hl                         ;

                        ld hl,LineBuffer                ;
                        ld (hl),$00                     ;
                        ld ix,AssParam1                 ;
DT_GetNextParam1        push hl                         ;
DT_GetNextParam         call Read                       ;
                        jr c DT_Fail                    ;

                        ld (ix),l                       ;
                        ld (ix+1),h                     ;
DT_GNP1                 inc ix                          ;
                        inc ix                          ;
                        jr DT_GetNextParam              ;

DT_Fail                 inc de                          ;
                        cp ','                          ;
                        jr z DT_GNP1                    ;

                        pop hl                          ;
                        or a                            ;
                        jr nz DT_GNP2                   ;

                        ld a,(LastCmdIndexOrAssFlags)   ;
                        cp HiddenCommandIndex           ;
                        ret z                           ;

                        ; Reset the hidden message for any command that doesn't show it

                        ld hl,sHiddenMsg2               ;
                        ld (HiddenMsgBase),hl           ;
                        ret                             ;

DT_GNP2                 call Rdstg                      ;
                        jr nc DT_GetNextParam1          ;

                        ret                             ;

AssHaveLineNumber       call Read                       ;
                        ret c                           ;

                        ld a,h                          ;
                        and l                           ;
                        cp $FF                          ;
                        scf                             ;
                        ret z                           ;

                        ld (AssParam1),hl               ;
                        ld a,(de)                       ;
                        cp $20                          ;
                        jr nz AHLN_NotSpace             ;

                        inc de                          ;
AHLN_NotSpace           ld hl,Buffer                    ;
                        ld c,$00                        ;
                        call TokeniseLine               ;
                        ld de,Buffer                    ;
                        sbc hl,de                       ;
                        ld (LineLength),hl              ;
                        ld de,(AssParam1)               ;
                        call FindLineNum_DE             ;
                        jr c AHLN_NotFound              ;

                        jr z AssReplaceLine             ;

                        xor a                           ;
AHLN_NotFound           ex af,af'                       ;
                        ld bc,(LineLength)              ;
                        ld a,b                          ;
                        or c                            ;
                        ret z                           ;

                        ex af,af'                       ;
                        jr z AHLN_NotFirstLine          ;

                        ld a,$00                        ;
                        ld (NoSourceFlag),A             ;

                        ld (hl),$FF                     ;
                        inc hl                          ;
                        ld (SourceEndAddr),hl           ;

                        ld (hl),$FF                     ;
                        inc hl                          ;

AHLN_NotFirstLine       dec hl                          ;
                        dec hl                          ;

                        inc bc                          ;
                        inc bc                          ;
                        inc bc                          ;

                        call InsertSpaceForLine         ;

                        ld (hl),e                       ;
                        inc hl                          ;
                        ld (hl),d                       ;
                        inc hl                          ;

CopyLineInBuffToHL      ex de,hl                        ;
CopyLineInBuffToDE      ld hl,Buffer                    ;
                        ld bc,(LineLength)              ;
                        inc bc                          ;
                        ldir                            ;

                        ret                             ;

AssReplaceLine          call FindLineLengthBC           ;
                        ld de,(LineLength)              ;
                        ld a,d                          ;
                        or e                            ;
                        jr z ARL_ReplaceWithEmpty       ;

                        ex de,hl                        ;
                        sbc hl,bc                       ;
                        jr z CopyLineInBuffToDE         ;

                        ex de,hl                        ;
                        ld b,d                          ;
                        ld c,e                          ;
                        jr c ARL_ReplaceShorter         ;

                        call InsertSpaceForLine         ;
                        jr CopyLineInBuffToHL           ;

ARL_ReplaceShorter      xor a                           ;
                        sub c                           ;
                        ld c,a                          ;
                        ld a,$00                        ;
                        sbc a,b                         ;
                        ld b,a                          ;
                        call RemoveSpaceFromSource      ;
                        jr CopyLineInBuffToHL           ;

ARL_ReplaceWithEmpty    dec hl                          ;
                        dec hl                          ;

                        inc bc                          ;
                        inc bc                          ;
                        inc bc                          ;

RemoveSpaceFromSource   push hl                         ;
                        push de                         ;

                        ld d,h                          ;
                        ld e,l                          ;
                        add hl,bc                       ;
                        ld bc,(SourceEndAddr)           ;
                        ld a,c                          ;
                        sub l                           ;
                        ld c,a                          ;
                        ld a,b                          ;
                        sbc a,h                         ;
                        ld b,a                          ;
                        inc bc                          ;
                        ldir                            ;
                        dec de                          ;
                        ld (SourceEndAddr),de           ;

                        pop de                          ;
                        pop hl                          ;
                        ret                             ;

InsertSpaceForLine      push de                         ;
                        ex de,hl                        ;
                        ld hl,(SourceEndAddr)           ;
                        add hl,bc                       ;
                        ld bc,$DF80                     ;
                        or a                            ;
                        sbc hl,bc                       ;
                        add hl,bc                       ;
                        jr c ISFL_IsRoom                ;

                        call PrintStringZ               ;
                        db "Out of memory !",cCR,0      ;

                        jp MainGetEntryLine             ;

ISFL_IsRoom             ex de,hl                        ;
                        ld bc,(SourceEndAddr)           ;
                        ld a,c                          ;
                        sub l                           ;
                        ld c,a                          ;
                        ld a,b                          ;
                        sbc a,h                         ;
                        ld b,a                          ;
                        inc bc                          ;
                        ld hl,(SourceEndAddr)           ;
                        ld (SourceEndAddr),de           ;
                        lddr                            ;
                        inc hl                          ;
                        pop de                          ;
                        ret                             ;

TL_NextCharLp1          inc hl                          ;
TL_NextCharLp           inc de                          ;
TokeniseLine            ld a,(de)                       ;
                        ld (hl),a                       ;
                        or a                            ;
                        ret z                           ;

                        cp $20                          ;
                        jr nz TL_NotSp                  ;

                        inc c                           ;
                        jr TL_NextCharLp                ;

TL_NotSp                dec c                           ;
                        jr z TL_NoSpaces                ;

                        inc c                           ;
                        jr z TL_HasSpace                ;

                        ld (hl),$0A                     ; Compressed space marker
                        inc hl                          ;
                        ld (hl),c                       ;
                        inc hl                          ;

                        jr TL_HasSpace                  ;

TL_NoSpaces             ld (hl),$20                     ;
                        inc hl                          ;
TL_HasSpace             ld bc,$7F00                     ;
                        push hl                         ;
                        ld hl,TokenDefnTable            ;
                        call Tokenize                   ;
                        pop hl                          ;
                        jr c TL_NotAToken               ;

                        ld (hl),b                       ;
                        inc hl                          ;
                        jr TokeniseLine                 ;

TL_NotAToken            ld a,(de)                       ;
                        ld (hl),a                       ;
                        call IfIdentChar_NC             ;
                        jr nc TL_NotTokenOrIdent        ;

                        call IfPfx_NC                   ;
                        jr nz TL_NextCharLp1            ;

TL_NotTokenOrIdent      inc de                          ;
                        inc hl                          ;
                        ld a,(de)                       ;
                        ld (hl),a                       ;
                        call IfIdentChar_NC             ;
                        jr nc TL_NotTokenOrIdent        ;

                        jr TokeniseLine                 ;

DeTok_SkipDEHL          inc de                          ;
DeTok_SkipHL            inc hl                          ;

DeTok_Lp                ld a,(hl)                       ;
                        cp $0A                          ; Compressed spaces?
                        jr z DeTok_CmpSP                ;

                        cp $80                          ; Tokens?
                        jr nc DeTok_HitToken            ;

LE294                   call DeTok_OutputA              ;
                        or a                            ;
                        jr nz DeTok_SkipDEHL            ;

                        ret                             ;

DeTok_HitToken          sub $7F                         ;
                        ld b,a                          ;

                        push hl                         ;
                        ld hl,TokenDefnTable            ;
                        call Token3                     ;
                        jr nc LE2AB                     ;
                        pop hl                          ;

                        ld a,(hl)                       ;
                        jr LE294                        ;

LE2AB                   ld a,(hl)                       ;
                        cp ' '                          ;
                        jr c LE2B7                      ;

                        call DeTok_OutputA              ;
                        inc de                          ;
                        inc hl                          ;
                        jr LE2AB                        ;

LE2B7                   pop hl                          ;
                        jr DeTok_SkipHL                 ;

DeTok_CmpSP             inc hl                          ;
                        ld b,(hl)                       ;

                        ld a,' '                        ;

DeTokRpt_Lp             call DeTok_OutputA              ;
                        inc de                          ;
                        djnz DeTokRpt_Lp                ;

                        jr DeTok_SkipHL                 ;

; Output the char in A to a buffer or the output data

DeTok_OutputA           bit 0,c                         ;
                        jr nz DeTok_OutputA_Plant       ;

                        ld (de),a                       ;
                        ret                             ;

DeTok_OutputA_Plant     or a                            ;
                        ret z                           ;

                        push af                         ;
                        call PlantByteA                 ;
                        pop af                          ;
                        ret                             ;

Tokenize                inc b                           ;
                        ld a,(de)                       ;
                        cp (hl)                         ;
                        ret c                           ;

                        jr z TokenSameStartChar         ;

Token1                  ld a,(hl)                       ;
                        inc hl                          ;
                        cp $20                          ;
                        jr nc Token1                    ;

                        cp $08                          ;
                        jr z Tokenize                   ;

                        cp $0A                          ; Compressed space marker
                        jr z Tokenize                   ;

                        scf                             ;
                        ret                             ;

TokenSameStartChar      push de                         ;
TokenCmpLp              inc de                          ;
                        inc hl                          ;
                        ld a,(de)                       ;
                        cp (hl)                         ;
                        jr z TokenCmpLp                 ;

                        ld a,(hl)                       ;
                        cp $08                          ;
                        jr z Token5                     ;

                        cp $0A                          ; Compressed space marker
                        jr nz Token4                    ;

                        ld a,(de)                       ;
                        call IfIdentChar_NC             ;
                        jr c Token5                     ;

Token4                  pop de                          ;
                        jr Token1                       ;

Token5                  pop hl                          ;
                        or a                            ;
                        ret                             ;

Token2                  ld a,(hl)                       ;
                        inc hl                          ;
                        cp $20                          ;
                        jr nc Token2                    ;

                        ld a,(hl)                       ;
                        cp $0C                          ;
                        scf                             ;
                        ret z                           ;

Token3                  djnz Token2                     ;

                        or a                            ;
                        ret                             ;

FindLineNum_DE          ld hl,(SourceStartAddr)         ;
                        ld a,(NoSourceFlag)             ;
                        add a,a                         ;
                        ret c                           ;

FLN_Lp                  ld c,(hl)                       ;
                        inc hl                          ;
                        ld b,(hl)                       ;
                        inc hl                          ;
                        ld a,b                          ;
                        and c                           ;
                        cp $FF                          ;
                        scf                             ;
                        ret z                           ;

                        ld a,b                          ;
                        cp d                            ;
                        jr c FLN_FindLineEnd            ;

                        ret nz                          ;

                        ld a,c                          ;
                        cp e                            ;
                        ret nc                          ;

FLN_FindLineEnd         xor a                           ;
FLN_FLELp               cp (hl)                         ;
                        inc hl                          ;
                        jr nz FLN_FLELp                 ;

                        jr FLN_Lp                       ;

FindLineLengthBC        xor a                           ;
                        push hl                         ;
                        ld b,a                          ;
                        ld c,a                          ;
                        cpir                            ;
                        sub c                           ;
                        ld c,a                          ;
                        ld a,$00                        ;
                        sbc a,b                         ;
                        ld b,a                          ;
                        dec bc                          ;
                        pop hl                          ;
                        ret                             ;

LE345                   ld hl,(AssParam3)               ;
                        ld (LFD09),hl                   ;

                        ld de,(AssParam1)               ;
                        call FindLineNum_DE             ;
                        jr nc LE362                     ;

                        ret                             ;

GotoNextLine            ld hl,(SrcLinePointer)          ;
                        ld c,(hl)                       ;
                        inc hl                          ;
                        ld b,(hl)                       ;
                        inc hl                          ;
                        ld a,b                          ;
                        and c                           ;
                        cp $FF                          ;
                        scf                             ;
                        ret z                           ;

LE362                   ex de,hl                        ;
                        ld hl,(AssParam2)               ;
                        or a                            ;
                        sbc hl,bc                       ;
                        ret c                           ;

                        ex de,hl                        ;
                        ld (AssParam1),bc               ;
                        ld de,Buffer                    ;
                        ld c,$00                        ;
                        call DeTok_Lp                   ;
                        inc hl                          ;
                        ld (SrcLinePointer),hl          ;
                        ret                             ;

LE37C                   ld hl,(AssParam1)               ;
                        ld a,cCLEARLINE                 ;
                        call PrintChar                  ;
                        call PrintDecimal               ;

                        ld hl,Buffer                    ;
LE38A                   ld a,(hl)                       ;
                        inc hl                          ;
                        or a                            ;
                        jr z LE394                      ;

                        call PrintChar                  ;
                        jr LE38A                        ;

LE394                   ld a,cCR                        ;
                        call PrintChar                  ;

LE399                   ld hl,(LFD09)                   ;
                        ld a,h                          ;
                        or l                            ;
                        ret z                           ;

                        dec hl                          ;
                        ld a,h                          ;
                        or l                            ;
                        ld (LFD09),hl                   ;
                        ret nz                          ;

                        call InputChar                  ;
                        cp cCR                          ;
                        scf                             ;
                        ret nz                          ;

                        ld hl,(AssParam3)               ;
                        ld (LFD09),hl                   ;
                        or a                            ;
                        ret                             ;

CmdInsert               pop hl                          ;
                        ld hl,(AssParam1)               ;
                        jp MainInsertLN                 ;

CmdList                 call LE345                      ;
LE3BF                   ccf                             ;
                        ret nc                          ;

                        call LE37C                      ;
                        ccf                             ;
                        ret nc                          ;

                        call GotoNextLine               ;
                        jr LE3BF                        ;

CmdRenumber             ld de,(AssParam3)               ;
                        call FindLineNum_DE             ;
                        ccf                             ;
                        ret nc                          ;

                        dec hl                          ;
                        dec hl                          ;
                        ld bc,(AssParam2)               ;
                        ld de,(AssParam1)               ;
CmdRenumberLp           ld (hl),e                       ;
                        inc hl                          ;
                        ld (hl),d                       ;
                        inc hl                          ;
                        xor a                           ;
CmdRenumber_FindEOL     cp (hl)                         ;
                        inc hl                          ;
                        jr nz CmdRenumber_FindEOL       ;

                        ld a,(hl)                       ;
                        inc hl                          ;
                        and (hl)                        ;
                        cp $FF                          ;
                        ret nc                          ;

                        dec hl                          ;
                        ex de,hl                        ;
                        add hl,bc                       ;
                        ret c                           ;

                        ex de,hl                        ;
                        jr CmdRenumberLp                ;

CmdDelete               ld de,(AssParam1)               ;
                        call FindLineNum_DE             ;
                        ccf                             ;
                        ret nc                          ;

                        dec hl                          ;
                        dec hl                          ;
                        push hl                         ;
                        ld de,(AssParam2)               ;
                        inc de                          ;
                        call FindLineNum_DE             ;
                        dec hl                          ;
                        dec hl                          ;
                        pop de                          ;
                        or a                            ;
                        sbc hl,de                       ;
                        ret z                           ;

                        ccf                             ;
                        ret nc                          ;

                        ex de,hl                        ;
                        ld b,d                          ;
                        ld c,e                          ;
                        jp RemoveSpaceFromSource        ;

CmdFind                 call LE345                      ;
CmdFind1                ccf                             ;
                        ret nc                          ;

                        ld de,LFDFF                     ;
FindCmpLine             ld hl,LineBuffer                ;
FindCmpLineLp           inc de                          ;
                        ld a,(de)                       ;
                        or a                            ;
                        jr z FindCmpLineEnd             ;

                        cp (hl)                         ;
                        jr nz FindCmpLineLp             ;

                        push de                         ;
LE42B                   inc hl                          ;
                        inc de                          ;
                        ld a,(hl)                       ;
                        or a                            ;
                        jr z LE43B                      ;

                        ld a,(de)                       ;
                        or a                            ;
                        jr z LE438                      ;

                        cp (hl)                         ;
                        jr z LE42B                      ;

LE438                   pop de                          ;
                        jr FindCmpLine                  ;

LE43B                   pop de                          ;
                        call LE37C                      ;
                        ccf                             ;
                        ret nc                          ;

FindCmpLineEnd          call GotoNextLine               ;
                        jr CmdFind1                     ;

; Start the code
CmdExecute              ld hl,(ExecuteAddr)             ;
                        jp (hl)                         ;

; Kill the current source
CmdNew                  ld a,$FF                        ;
                        jp SetNoSourceFlag              ;

; Recover an old source
CmdOld                  ld a,$00                        ;
SetNoSourceFlag         ld (NoSourceFlag),A             ;

                        ld hl,(AssParam1)               ;
                        ld (SourceStartAddr),hl         ;

                        jp MainWarm                     ;

; List symbols?
CmdSymbol               ld hl,(AssParam1)               ;
                        ld (AssParam3),hl               ;
                        ld (LFD09),hl                   ;
                        ld de,(SymbolTableBase)         ;
CmdSymbolLp             ld hl,LabelBuffer               ;
                        or a                            ;
                        sbc hl,de                       ;
                        ret z                           ;

                        ld b,$0E                        ;
                        ld a,(de)                       ;
                        ld c,a                          ;
LE475                   ld a,(de)                       ;
                        and $7F                         ;
                        inc de                          ;
                        call PrintChar                  ;
                        djnz LE475                      ;

                        ld a,(de)                       ;
                        ld l,a                          ;
                        inc de                          ;
                        ld a,(de)                       ;
                        ld h,a                          ;
                        inc de                          ;
                        call PrintStringZ               ;
                        db "= #",0                      ;
                        bit 7,c                         ;
                        jr nz LE49C                     ;

                        call PrintStringZ               ;
                        db cLEFT,"Undef ",0             ;
                        jr LE49F                        ;

LE49C                   call PrintHexWord               ;
LE49F                   ld a,$20                        ;
                        call PrintChar                  ;
                        ld a,cCR                        ;
                        call PrintChar                  ;
                        call LE399                      ;
                        ccf                             ;
                        jr c CmdSymbolLp                ;

                        ret                             ;

IfLetterUpperOnly_NC    cp 'A'                          ;
                        ret c                           ;

                        cp 'Z'+1                        ;
                        ccf                             ;
                        ret                             ;

IfNumber_NC             cp '0'                          ;
                        ret c                           ;

                        cp '9'+1                        ;
                        ccf                             ;
                        ret                             ;

IfIdentChar_NC          call IfNumber_NC                ;
                        ret nc                          ;

IfLetterEitherCase_NC   call IfLetterUpperOnly_NC       ;
                        ret nc                          ;

                        cp 'a'                          ;
                        ret c                           ;

                        cp 'z'+1                        ;
                        ccf                             ;
                        ret                             ;

IfPfx_NC                cp '"'                          ;
                        ret z                           ;

                        cp '#'                          ;
                        ret                             ;

IfHexChar_NC            sub '0'                         ;
                        ret c                           ;

                        cp 10                           ;
                        ccf                             ;
                        ret nc                          ;

                        sub $07                         ;
                        cp 10                           ;
                        ret c                           ;

                        cp $11                          ;
                        ccf                             ;
                        ret                             ;

           zeusdatafollows PrintStringZ    ; Tell Zeus this routine expects data to follow (avoids flow warnings)

; Print the bytes following the call in memory, until a zero
PrintStringZ            ex (sp),hl                      ;
                        push af                         ;
PrintStringZLp          ld a,(hl)                       ;
                        inc hl                          ;
                        or a                            ;
                        jr z PrintStringZX              ;

                        call PrintChar                  ;
                        jr PrintStringZLp               ;

PrintStringZX           pop af                          ;
                        ex (sp),hl                      ;
                        ret                             ;

Skip2                   inc de                          ;
Skip1                   inc de                          ;
Skip                    ld a,(de)                       ;
                        cp ' '                          ;
                        jr z Skip1                      ;

                        cp $0A                          ; Compressed space marker
                        jr z Skip2                      ;

                        or a                            ;
                        ret                             ;

SkipT                   inc de                          ;
SkipSpc                 call Skip                       ;
                        ret z                           ;

                        cp ':'                          ;
                        ret z                           ;

                        cp ';'                          ;
                        ret                             ;

Rdstg                   ld b,a                          ;
                        ld c,$00                        ;
Rdstg1                  ld a,(de)                       ;
                        inc de                          ;
                        cp b                            ;
                        jr z Rdstg3                     ;

                        or a                            ;
                        jr z Rdstg4                     ;

                        cp $20                          ;
                        jr z Rdstg2                     ;

                        ld c,$FF                        ;
Rdstg2                  inc c                           ;
                        ld (hl),a                       ;
                        inc hl                          ;
                        jr Rdstg1                       ;

Rdstg3                  ld (hl),$00                     ;
                        inc hl                          ;
                        ret                             ;

Rdstg4                  ld (hl),a                       ;
                        cp c                            ;
                        scf                             ;
                        ret z                           ;

                        dec hl                          ;
                        dec c                           ;
                        jr Rdstg4                       ;

Read                    ld h,$00                        ;
                        call Skip                       ;
                        cp '#'                          ;
                        jr z ReadHex                    ;

                        call IfNumber_NC                ;
                        ret c                           ;

                        sub '0'                         ;
                        ld l,a                          ;
Read1                   inc de                          ;
                        ld a,(de)                       ;
                        call IfNumber_NC                ;
                        ccf                             ;
                        ret nc                          ;

                        sub '0'                         ;
                        ld b,h                          ;
                        ld c,l                          ;
                        add hl,hl                       ;
                        ret c                           ;

                        add hl,hl                       ;
                        ret c                           ;

                        add hl,bc                       ;
                        ret c                           ;

                        add hl,hl                       ;
                        ret c                           ;

                        add a,l                         ;
                        ld l,a                          ;
                        jr nc Read1                     ;

                        inc h                           ;
                        jr nz Read1                     ;

                        ret                             ;

ReadHex                 inc de                          ;
                        ld a,(de)                       ;
                        call IfHexChar_NC               ;
                        ret c                           ;

                        ld l,a                          ;
Read3                   inc de                          ;
                        ld a,(de)                       ;
                        call IfHexChar_NC               ;
                        ccf                             ;
                        ret nc                          ;

                        add hl,hl                       ;
                        ret c                           ;

                        add hl,hl                       ;
                        ret c                           ;

                        add hl,hl                       ;
                        ret c                           ;

                        add hl,hl                       ;
                        ret c                           ;

                        or l                            ;
                        ld l,a                          ;
                        jr Read3                        ;

PrintHexWord            ld a,h                          ;
                        call PrintHex8                  ;
                        ld a,l                          ;
                        call PrintHex8                  ;
                        ld a,$20                        ;
                        call PrintChar                  ;
                        ret                             ;

PrintHex8               push af                         ;
                        rra                             ;
                        rra                             ;
                        rra                             ;
                        rra                             ;
                        and $0F                         ;
                        call PrintHex4                  ;
                        call PrintChar                  ;
                        pop af                          ;
                        and $0F                         ;
                        call PrintHex4                  ;
                        call PrintChar                  ;
                        ret                             ;

PrintHex4               add a,'0'                       ;
                        cp '9'+1                        ;
                        ret c                           ;

                        add a,$07                       ;
                        ret                             ;

PrintDecNZ              ld c,1                          ;
                        jp PD_HaveLZFlag                ;

PrintDecimal            ld c,0                          ;

PD_HaveLZFlag           ld de,10000                     ;
                        call PD_Digit                   ;

                        ld de,1000                      ;
                        call PD_Digit                   ;

                        ld de,100                       ;
                        call PD_Digit                   ;

                        ld de,10                        ;
                        call PD_Digit                   ;

                        ld a,l                          ;
                        add a,'0'                       ;
                        call PrintChar                  ;

                        ld a,' '                        ;
                        call PrintChar                  ;
                        ret                             ;

PD_Digit                xor a                           ;
PD_Digit_Lp             inc a                           ;
                        sbc hl,de                       ;
                        jr nc PD_Digit_Lp               ;

                        add hl,de                       ;
                        cp c                            ;
                        ret z                           ;

                        dec c                           ;
                        add a,'0'-1                     ;
                        call PrintChar                  ;
                        ret                             ;

DeclareLabel            ld a,(de)                       ;
                        call IfLetterEitherCase_NC      ;
                        jp c Error0                     ;

                        ld hl,LabelBuffer               ;
                        ld b,14                         ;
DL_GetLp                ld (hl),a                       ;
                        inc de                          ;
                        inc hl                          ;
                        ld a,(de)                       ;
                        call IfIdentChar_NC             ;
                        jr c DL_GetDone                 ;

                        djnz DL_GetLp                   ;

                        jp Error1                       ;

DL_GetDone              ex af,af'                       ;
DL_FillSpcLp            ld (hl),' '                     ;
                        inc hl                          ;
                        djnz DL_FillSpcLp               ;

                        push de                         ;
                        ld hl,LabelBuffer               ;
                        ld de,(SymbolTableBase)         ;
                        jr DL_FNS_Next                  ;

DL_MoveToNextSym        ld hl,LabelBuffer               ;
DL_FindNextSymLp        ld a,e                          ;
                        or $0F                          ;
                        ld e,a                          ;
                        inc de                          ;
DL_FNS_Next             ld a,(de)                       ;
                        and $7F                         ;
                        cp (hl)                         ;
                        jr nz DL_FindNextSymLp          ;

                        ld b,13                         ;
DL_CompareLp            inc hl                          ;
                        inc de                          ;
                        ld a,(de)                       ;
                        cp (hl)                         ;
                        jr nz DL_MoveToNextSym          ;

                        djnz DL_CompareLp               ;

                        sbc hl,de                       ;
                        jr nz DL_Done                   ;

                        ex af,af'                       ;
                        ld hl,(SymbolTableBase)         ;
                        ld bc,(SourceEndAddr)           ;
                        or a                            ;
                        sbc hl,bc                       ;
                        ld bc,32                        ;
                        or a                            ;
                        sbc hl,bc                       ;
                        jr nc DL_EnoughRoom             ;

                        call PrintStringZ               ;
                        db "Out of symbol space !",cCR,0;
                        jp MainGetEntryLine             ;

DL_EnoughRoom           ld hl,(SymbolTableBase)         ;
                        dec hl                          ;
                        dec hl                          ;
                        dec hl                          ;
                        ex de,hl                        ;
                        ld bc,14                        ;
                        lddr                            ;
                        inc de                          ;
                        ld (SymbolTableBase),de         ;
DL_Done                 ld a,e                          ;
                        and $F0                         ;
                        ld e,a                          ;
                        push de                         ;
                        pop ix                          ;
                        pop de                          ;
                        ex af,af'                       ;
                        ret                             ;

IsThisAFlag_Z           ld b,$00                        ;
                        cp tNZ                          ;
                        ret z                           ;

                        inc b                           ;
                        cp tZ                           ;
                        ret z                           ;

                        inc b                           ;
                        cp tNC                          ;
                        ret z                           ;

                        inc b                           ;
                        cp tC                           ;
                        ret z                           ;

                        inc b                           ;
                        cp tPO                          ;
                        ret z                           ;

                        cp tNV                          ;
                        ret z                           ;

                        inc b                           ;
                        cp tPE                          ;
                        ret z                           ;

                        cp tV                           ;
                        ret z                           ;

                        inc b                           ;
                        cp tP                           ;
                        ret z                           ;

                        inc b                           ;
                        cp tM                           ;
                        ret                             ;

; H us the token for the variable reg pair
IsThisA16BitReg1_Z      ld b,$00                        ;
                        cp tBC                          ;
                        ret z                           ;

                        inc b                           ;
                        cp tDE                          ;
                        ret z                           ;

                        inc b                           ;
                        call IsHL_IX_IY                 ;
                        ret z                           ;

                        inc b                           ;
                        cp h                            ;
                        ret                             ;

IsThisA16BitReg2_Z      ld b,$00                        ;
                        cp tBC                          ;
                        ret z                           ;

                        inc b                           ;
                        cp tDE                          ;
                        ret z                           ;

                        inc b                           ;
                        cp h                            ;
                        ret z                           ;

                        inc b                           ;
                        cp tSP                          ;
                        ret                             ;

IsThisAn8BitTarget      ld b,$00                        ;
                        cp tB                           ;
                        ret z                           ;

                        inc b                           ;
                        cp tC                           ;
                        ret z                           ;

                        inc b                           ;
                        cp tD                           ;
                        ret z                           ;

                        inc b                           ;
                        cp tE                           ;
                        ret z                           ;

                        inc b                           ;
                        cp tH                           ;
                        ret z                           ;

                        inc b                           ;
                        cp tL                           ;
                        ret z                           ;

                        inc b                           ;
                        cp '('                          ;
                        jr z LE6D7                      ;

                        inc b                           ;
                        cp tA                           ;
                        ret z                           ;

                        scf                             ;
                        ccf                             ;
                        ret                             ;

LE6D7                   call Skip1                      ;
                        call IsHL_IX_IY                 ;
                        jr nz LE714                     ;

                        jr c LE6EA                      ;

                        call Skip1                      ;
                        cp ')'                          ;
                        ret z                           ;

                        jp Error2                       ;

LE6EA                   ld l,$00                        ;
                        call Skip1                      ;
                        cp '+'                          ;
                        jr nz LE6FD                     ;

                        call Evalbs                     ;
                        cp $80                          ;
                        jr c LE70B                      ;

                        jp Error3                       ;

LE6FD                   cp '-'                          ;
                        jr nz LE70F                     ;

                        call Evalbs                     ;
                        cp $81                          ;
                        jp nc Error3                    ;

                        neg                             ;
LE70B                   ld l,a                          ;
                        call Skip                       ;
LE70F                   cp ')'                          ;
                        jp nz Error2                    ;

LE714                   scf                             ;
                        ret                             ;

IsHL_IX_IY              cp tHL                          ;
                        ret z                           ;

                        cp tIX                          ;
                        jr nz IsThisIY_Z                ;

                        ld a,$DD                        ;
                        jp PlantByteA                   ;

IsThisIY_Z              cp tIY                          ;
                        ret nz                          ;

                        ld a,$FD                        ;
                        jp PlantByteA                   ;

IsThisAn8BitReg_Z       ld b,$00                        ;
                        cp tB                           ;
                        ret z                           ;

                        inc b                           ;
                        cp tC                           ;
                        ret z                           ;

                        inc b                           ;
                        cp tD                           ;
                        ret z                           ;

                        inc b                           ;
                        cp tE                           ;
                        ret z                           ;

                        inc b                           ;
                        cp tH                           ;
                        ret z                           ;

                        inc b                           ;
                        cp tL                           ;
                        ret z                           ;

                        ld b,$07                        ;
                        cp tA                           ;
                        ret                             ;

SkipComma               call Skip1                      ;
                        cp ','                          ;
                        jp nz Error4                    ;

                        call Skip1                      ;
                        ret                             ;

SkipAndPlant_BorC       inc de                          ;
Plant_BorC              ld a,b                          ;
                        or c                            ;
                        jp PlantByteA                   ;

SkipAndPlant_B_SHL3_orC inc de                          ;
Plant_B_SHL3_orC        ld a,b                          ;
                        add a,a                         ;
                        add a,a                         ;
                        add a,a                         ;
                        or c                            ;
                        jp PlantByteA                   ;

SkipAndPlant_B_SHL4_orC inc de                          ;
Plant_B_SHL4_orC        ld a,b                          ;
                        add a,a                         ;
                        add a,a                         ;
                        add a,a                         ;
                        add a,a                         ;
                        or c                            ;
                        jp PlantByteA                   ;

dwSkipLDI_Etc           inc de                          ;

dwLDI_Etc               ld a,$ED                        ;
                        call PlantByteA                 ;

dwFixedByte_C           ld a,c                          ;
                        jp PlantByteA                   ;

dwRST                   call Evalb                      ;
                        ld b,a                          ;
                        and c                           ;
                        jp z Plant_BorC                 ;

                        jp Error5                       ;

dwRET_Etc               jr nz LE789                     ;

                        ld a,$C9                        ;
                        jp PlantByteA                   ;

LE789                   call IsThisAFlag_Z              ;
                        jp z SkipAndPlant_B_SHL3_orC    ;

                        jp Error5                       ;

dwPUSH                  ld h,tAF                        ;
                        call IsThisA16BitReg1_Z         ;
                        jp z SkipAndPlant_B_SHL4_orC    ;

                        jp Error5                       ;

dwBIT_Etc               call Evalb                      ;
                        cp $08                          ;
                        jp nc Error3                    ;

                        add a,a                         ;
                        add a,a                         ;
                        add a,a                         ;
                        or c                            ;
                        ld c,a                          ;
                        call Skip                       ;
                        cp ','                          ;
                        jp nz Error4                    ;

                        call Skip1                      ;
dwRL_Etc                call IsThisAn8BitTarget         ;
                        jp nz Error5                    ;

                        jr c Plant_CB_BorC              ;

                        ld a,$CB                        ;
                        call PlantByteA                 ;
                        jp SkipAndPlant_BorC            ;

Plant_CB_BorC           ld a,$CB                        ;
                        call PlantByteA                 ;
                        ld a,l                          ;
                        call PlantByteA                 ;
                        jp SkipAndPlant_BorC            ;

dwAND_Etc               call IsThisAn8BitTarget         ;
                        jr nz LE7E0                     ;

                        jp nc SkipAndPlant_BorC         ;

                        call SkipAndPlant_BorC          ;
                        ld a,l                          ;
                        jp PlantByteA                   ;

LE7E0                   jp c Error5                     ;

                        call Evalb                      ;
                        ld l,a                          ;
                        ld a,c                          ;
                        or $46                          ;
                        call PlantByteA                 ;
                        ld a,l                          ;
                        jp PlantByteA                   ;

dwADD_Etc               cp tA                           ;
                        jr nz LE7FA                     ;

                        call SkipComma                  ;
                        jr dwAND_Etc                    ;

LE7FA                   ld h,a                          ;
                        bit 3,c                         ;
                        jr nz LE813                     ;

                        ld c,$09                        ;
                        call IsHL_IX_IY                 ;
                        jp nz Error5                    ;

                        call SkipComma                  ;
                        call IsThisA16BitReg2_Z         ;
                        jp z SkipAndPlant_B_SHL4_orC    ;

                        jp Error5                       ;

LE813                   ld b,a                          ;
                        ld a,$ED                        ;
                        call PlantByteA                 ;
                        ld a,$42                        ;
                        bit 4,c                         ;
                        jr nz LE821                     ;

                        or $08                          ;
LE821                   ld c,a                          ;
                        ld a,b                          ;
                        cp tHL                          ;
                        jp nz Error5                    ;

                        call SkipComma                  ;
                        call IsThisA16BitReg2_Z         ;
                        jp z SkipAndPlant_B_SHL4_orC    ;

                        jp Error5                       ;

dwDEC_Etc               ld h,tSP                        ;
                        call IsThisA16BitReg1_Z         ;
                        jr nz LE847                     ;

                        ld a,$03                        ;
                        bit 0,c                         ;
                        jr z LE843                      ;

                        or $08                          ;
LE843                   ld c,a                          ;
                        jp SkipAndPlant_B_SHL4_orC      ;

LE847                   call IsThisAn8BitTarget         ;
                        jp nz Error5                    ;

                        jp nc SkipAndPlant_B_SHL3_orC   ;

                        call SkipAndPlant_B_SHL3_orC    ;
                        ld a,l                          ;
                        jp PlantByteA                   ;

dwCALL_Etc              call IsThisAFlag_Z              ;
                        jr nz dwCALL                    ;

                        call Plant_B_SHL3_orC           ;
                        call SkipComma                  ;
                        jr Plant_Word                   ;

dwCALL                  ld a,$CD                        ;
dwPlant_A_Addr          call PlantByteA                 ;

Plant_Word              call Eval                       ;
LE86C                   ld a,l                          ;
                        call PlantByteA                 ;
                        ld a,h                          ;
                        jp PlantByteA                   ;

dwJP_Etc                call IsThisAFlag_Z              ;
                        jr nz dwJP                      ;

                        call Plant_B_SHL3_orC           ;
                        call SkipComma                  ;
                        jr Plant_Word                   ;

dwJP                    cp '('                          ;
                        jr z dwJP_Bra                   ;

                        ld a,$C3                        ;
                        jr dwPlant_A_Addr               ;

dwJP_Bra                call Skip1                      ;
                        call IsHL_IX_IY                 ;
                        jp nz Error5                    ;

                        call Skip1                      ;
                        cp ')'                          ;
                        jp nz Error2                    ;

                        inc de                          ;
                        ld a,$E9                        ;
                        jp PlantByteA                   ;

dwJR_Etc                call IsThisAFlag_Z              ;
                        jr nz dwJR                      ;

                        ld a,b                          ;
                        cp $04                          ;
                        jp nc Error5                    ;

                        call Plant_B_SHL3_orC           ;
                        call SkipComma                  ;
                        jr Handle_JR_Disp               ;

dwJR                    ld a,$18                        ;
                        call PlantByteA                 ;

Handle_JR_Disp          ld bc,(LineLength)              ;
                        ld a,c                          ;
                        sub $7E                         ;
                        ld c,a                          ;
                        jr nc Handle_JR_Disp_1          ;

                        dec b                           ;
Handle_JR_Disp_1        call Eval                       ;
                        jr c Handle_JR_Disp_X           ;

                        sbc hl,bc                       ;
                        ld a,h                          ;
                        or a                            ;
                        jp nz Error3                    ;

                        ld a,l                          ;
                        xor $80                         ;
Handle_JR_Disp_X        jp PlantByteA                   ;

dwDJNZ                  ld a,c                          ;
                        call PlantByteA                 ;

                        jr Handle_JR_Disp               ;

dwDEFB                  call Evalb                      ;
                        call PlantByteA                 ;
                        call SkipSpc                    ;
                        ret z                           ;

                        cp ','                          ;
                        jp nz Error4                    ;

                        inc de                          ;
                        jr dwDEFB                       ;

dwDEFW                  call Plant_Word                 ;
                        call SkipSpc                    ;
                        ret z                           ;

                        cp ','                          ;
                        jp nz Error4                    ;

                        inc de                          ;
                        jr dwDEFW                       ;

dwDEFS                  call Eval                       ;
                        jp c Error9                     ;

                        ld bc,(AssPC)                   ;
                        add hl,bc                       ;
                        ld (AssPC),hl                   ;
                        ret                             ;

dwORG                   call Eval                       ;
                        jp c Error9                     ;

                        ld (AssPC),hl                   ;
                        ret                             ;

dwDISP                  call Eval                       ;
                        ld (AssDISP),hl                 ;
                        ret                             ;

dwENT                   ld hl,(AssPC)                   ;
                        ld (ExecuteAddr),hl             ;
                        ret                             ;

dwEX                    cp tDE                          ;
                        jr z LE94F                      ;

                        cp tAF                          ;
                        jr z LE95D                      ;

                        cp '('                          ;
                        jp nz Error7                    ;

                        call Skip1                      ;
                        cp tSP                          ;
                        jp nz Error5                    ;

                        call Skip1                      ;
                        cp ')'                          ;
                        jp nz Error2                    ;

                        call SkipComma                  ;
                        call IsHL_IX_IY                 ;
                        jp nz Error5                    ;

                        ld a,$E3                        ;
                        inc de                          ;
                        jp PlantByteA                   ;

LE94F                   call SkipComma                  ;
                        cp tHL                          ;
                        jp nz Error5                    ;

                        ld a,$EB                        ;
                        inc de                          ;
                        jp PlantByteA                   ;

LE95D                   call SkipComma                  ;
                        cp tAFd                         ;
                        jp nz Error5                    ;

                        ld a,$08                        ;
                        inc de                          ;
                        jp PlantByteA                   ;

dwIN_Etc                cp tA                           ;
                        jr nz LE997                     ;

                        call SkipComma                  ;
                        cp '('                          ;
                        jp nz Error7                    ;

                        call Skip1                      ;
                        cp tC                           ;
                        jr nz LE982                     ;

                        ld b,$07                        ;
                        jr LE9AD                        ;

LE982                   ld a,$DB                        ;
                        call PlantByteA                 ;
                        call Evalb                      ;
                        call PlantByteA                 ;
                        call Skip                       ;
                        cp ')'                          ;
                        jp nz Error2                    ;

                        inc de                          ;
                        ret                             ;

LE997                   call IsThisAn8BitReg_Z          ;
                        jp nz Error5                    ;

                        call SkipComma                  ;
                        cp '('                          ;
                        jp nz Error7                    ;

                        call Skip1                      ;
                        cp tC                           ;
                        jp nz Error5                    ;

LE9AD                   call Skip1                      ;
                        cp ')'                          ;
                        jp nz Error2                    ;

                        ld a,$ED                        ;
                        call PlantByteA                 ;
                        jp SkipAndPlant_B_SHL3_orC      ;

dwOUT_Etc               cp '('                          ;
                        jp nz Error7                    ;

                        call Skip1                      ;
                        cp tC                           ;
                        jr z LE9E6                      ;

                        ld a,$D3                        ;
                        call PlantByteA                 ;
                        call Evalb                      ;
LE9D1                   call PlantByteA                 ;
                        call Skip                       ;
                        cp ')'                          ;
                        jp nz Error2                    ;

LE9DC                   call SkipComma                  ;
                        cp tA                           ;
                        jp nz Error5                    ;

                        inc de                          ;
                        ret                             ;

LE9E6                   call Skip1                      ;
                        cp ')'                          ;
                        jp nz Error2                    ;

                        ld a,$ED                        ;
                        call PlantByteA                 ;
                        call SkipComma                  ;
                        call IsThisAn8BitReg_Z          ;
                        jp z SkipAndPlant_B_SHL3_orC    ;

                        jp Error5                       ;

dwIM                    call Evalb                      ;
                        cp $03                          ;
                        jp nc Error3                    ;

                        ld l,a                          ;
                        ld a,$ED                        ;
                        call PlantByteA                 ;
                        ld a,$46                        ;
                        dec l                           ;
                        jr z dwIM_1                     ;

                        dec l                           ;
                        jr nz dwIM_X                    ;

                        or $08                          ;
dwIM_1                  or $10                          ;
dwIM_X                  jp PlantByteA                   ;

dwLD                    cp tA                           ;
                        jp nz LEA7C                     ;

                        call SkipComma                  ;
                        ld c,$78                        ;
                        call IsThisAn8BitTarget         ;
                        jr nz LEA35                     ;

                        jp nc SkipAndPlant_BorC         ;

                        call SkipAndPlant_BorC          ;
                        ld a,l                          ;
                        jp PlantByteA                   ;

LEA35                   jr nc LEA5F                     ;

                        cp tBC                          ;
                        jr nz LEA3F                     ;

                        ld a,$0A                        ;
                        jr LEA51                        ;

LEA3F                   cp tDE                          ;
                        jr nz LEA47                     ;

                        ld a,$1A                        ;
                        jr LEA51                        ;

LEA47                   ld a,$3A                        ;
LEA49                   call PlantByteA                 ;
LEA4C                   call Plant_Word                 ;
                        jr LEA55                        ;

LEA51                   inc de                          ;
                        call PlantByteA                 ;
LEA55                   call Skip                       ;
                        cp ')'                          ;
                        jp nz Error2                    ;

                        inc de                          ;
                        ret                             ;

LEA5F                   cp tI                           ;
                        jr z LEA72                      ;

                        cp tR                           ;
                        jr z LEA77                      ;

                        ld a,$3E                        ;
                        call PlantByteA                 ;
                        call Evalb                      ;
                        jp PlantByteA                   ;

LEA72                   ld c,$57                        ;
                        jp dwSkipLDI_Etc                ;

LEA77                   ld c,$5F                        ;
                        jp dwSkipLDI_Etc                ;

LEA7C                   ld h,tSP                        ;
                        call IsThisA16BitReg1_Z         ;
                        jr nz LEAB8                     ;

                        call SkipComma                  ;
                        cp '('                          ;
                        jr nz LEA9E                     ;

                        inc de                          ;
                        ld a,b                          ;
                        cp $02                          ;
                        ld a,$2A                        ;
                        jr z LEA49                      ;

                        ld a,$ED                        ;
                        call PlantByteA                 ;
                        ld c,$4B                        ;
                        call Plant_B_SHL4_orC           ;
                        jr LEA4C                        ;

LEA9E                   ld l,a                          ;
                        ld a,b                          ;
                        cp $03                          ;
                        ld a,l                          ;
                        jr nz LEAB0                     ;

                        call IsHL_IX_IY                 ;
                        jr nz LEAB0                     ;

                        ld a,$F9                        ;
                        inc de                          ;
                        jp PlantByteA                   ;

LEAB0                   ld c,$01                        ;
                        call Plant_B_SHL4_orC           ;
                        jp Plant_Word                   ;

LEAB8                   call IsThisAn8BitTarget         ;
                        jr nz LEB07                     ;

                        ld a,b                          ;
                        push af                         ;
                        add a,a                         ;
                        add a,a                         ;
                        add a,a                         ;
                        or c                            ;
                        ld c,a                          ;
                        pop af                          ;
                        jr c LEADD                      ;

                        cp $06                          ;
                        jr z LEAF0                      ;

                        call SkipComma                  ;
                        call IsThisAn8BitTarget         ;
                        jr nz LEAF9                     ;

                        jp nc SkipAndPlant_BorC         ;

LEAD6                   call SkipAndPlant_BorC          ;
                        ld a,l                          ;
                        jp PlantByteA                   ;

LEADD                   call SkipComma                  ;
                        call IsThisAn8BitReg_Z          ;
                        jr z LEAD6                      ;

                        ld a,c                          ;
                        and $3F                         ;
                        or $06                          ;
                        call PlantByteA                 ;
                        ld a,l                          ;
                        jr LEAFE                        ;

LEAF0                   call SkipComma                  ;
                        call IsThisAn8BitReg_Z          ;
                        jp z SkipAndPlant_BorC          ;

LEAF9                   ld a,c                          ;
                        and $3F                         ;
                        or $06                          ;
LEAFE                   call PlantByteA                 ;
                        call Evalb                      ;
                        jp PlantByteA                   ;

LEB07                   jr c LEB1E                      ;

                        cp tI                           ;
                        jr nz LEB15                     ;

                        ld c,$47                        ;
LEB0F                   call dwLDI_Etc                  ;
                        jp LE9DC                        ;

LEB15                   cp tR                           ;
                        ld c,$4F                        ;
                        jr z LEB0F                      ;

                        jp Error5                       ;

LEB1E                   cp tBC                          ;
                        jr nz LEB28                     ;

                        ld a,$02                        ;
                        inc de                          ;
                        jp LE9D1                        ;

LEB28                   cp tDE                          ;
                        jr nz LEB32                     ;

                        ld a,$12                        ;
                        inc de                          ;
                        jp LE9D1                        ;

LEB32                   call Eval                       ;
                        call Skip                       ;
                        cp ')'                          ;
                        jp nz Error2                    ;

                        call SkipComma                  ;
                        cp tA                           ;
                        jr nz LEB4D                     ;

                        ld a,$32                        ;
LEB46                   call PlantByteA                 ;
LEB49                   inc de                          ;
                        jp LE86C                        ;

LEB4D                   push hl                         ;
                        ld h,tSP                        ;
                        call IsThisA16BitReg1_Z         ;
                        pop hl                          ;
                        jp nz Error5                    ;

                        ld a,b                          ;
                        cp $02                          ;
                        ld a,$22                        ;
                        jp z LEB46                      ;

                        ld a,$ED                        ;
                        call PlantByteA                 ;
                        ld c,$43                        ;
                        call Plant_B_SHL4_orC           ;
                        jp LEB49                        ;

dwDEFM                  ret z                           ;

                        ld hl,LineBuffer                ;
                        inc de                          ;
                        call Rdstg                      ;
                        jr nc LEB77                     ;

                        dec de                          ;
LEB77                   push de                         ;
                        ld hl,LineBuffer                ;
                        ld c,$FF                        ;
                        call DeTok_Lp                   ;
                        pop de                          ;
                        ret                             ;

LEB82                   ld hl,LastCmdIndexOrAssFlags    ;
                        or a                            ;
                        dec (hl)                        ;
                        ret nz                          ;

                        ld (hl),$03                     ;
                        jr LEB9B                        ;

CmdAssemble             ld hl,LabelBuffer               ;
                        ld (SymbolTableBase),hl         ;
                        ld hl,(AssParam1)               ;
                        ld (AssParam3),hl               ;
                        ld (LFD09),hl                   ;
LEB9B                   ld hl,(SourceEndAddr)           ;
                        inc hl                          ;
                        ld (AssPC),hl                   ;
                        ld hl,$0000                     ;
                        ld (AssDISP),hl                 ;
                        ld hl,AssShowError              ;
                        ld (ExecuteAddr),hl             ;
                        ld de,(SourceStartAddr)         ;
LEBB2                   ld a,(de)                       ;
                        ld c,a                          ;
                        inc de                          ;
                        ld a,(de)                       ;
                        ld b,a                          ;
                        inc de                          ;
                        and c                           ;
                        cp $FF                          ;
                        jr z LEB82                      ;

                        ld (AssParam1),bc               ;
                        ld (SrcLinePointer),de          ;
LEBC5                   ld hl,(AssPC)                   ;
                        ld (LineLength),hl              ;
                        call SkipSpc                    ;
                        jr nz LEBE4                     ;

LEBD0                   inc de                          ;
                        or a                            ;
                        jr z LEBB2                      ;

                        cp $3A                          ;
                        jr z LEBC5                      ;

                        cp $3B                          ;
                        jp nz Error0                    ;

LEBDD                   ld a,(de)                       ;
                        or a                            ;
                        jr z LEBD0                      ;

                        inc de                          ;
                        jr LEBDD                        ;

LEBE4                   sub $80                         ;
                        jr nc LEC35                     ;

                        call DeclareLabel               ;
                        jr nc LEBFC                     ;

                        ld a,(LastCmdIndexOrAssFlags)   ;
                        cp $02                          ;
                        jp nc LEBFC                     ;

                        bit 7,(ix)                      ;
                        jp nz Error6                    ;

LEBFC                   call Skip                       ;
                        sub $80                         ;
                        jp c Error0                     ;

                        cp $20                          ;
                        jr nz LEC19                     ;

                        call Evals                      ;
                        jr c LEC4C                      ;

LEC0D                   ld (ix+14),l                    ;
                        ld (ix+15),h                    ;
                        set 7,(ix)                      ;
                        jr LEC4C                        ;

LEC19                   cp $3F                          ;
                        jr nz LEC28                     ;

                        call Evals                      ;
                        jp c Error9                     ;

                        ld (AssPC),hl                   ;
                        jr LEC0D                        ;

LEC28                   ld hl,(AssPC)                   ;
                        ld (ix+14),l                    ;
                        ld (ix+15),h                    ;
                        set 7,(ix)                      ;
LEC35                   ld hl,LEC4C                     ;
                        push hl                         ;
                        ld c,a                          ;
                        ld b,$00                        ;
                        ld hl,TokenHandlerTable         ;
                        add hl,bc                       ;
                        add hl,bc                       ;
                        add hl,bc                       ;
                        ld c,(hl)                       ;
                        inc hl                          ;
                        ld a,(hl)                       ;
                        inc hl                          ;
                        ld h,(hl)                       ;
                        ld l,a                          ;
                        call SkipT                      ;
                        jp (hl)                         ;

LEC4C                   call Skip                       ;
                        jp LEBD0                        ;

PlantByteA              exx                             ;
                        ld b,a                          ;
                        ld a,(LastCmdIndexOrAssFlags)   ;
                        cp $03                          ;
                        jr nz DontPlant                 ;

                        ld hl,(AssPC)                   ;
                        ld de,(AssDISP)                 ;
                        add hl,de                       ;
                        ld (hl),b                       ;

DontPlant               ld hl,(AssPC)                   ;
                        inc hl                          ;
                        ld (AssPC),hl                   ;

                        exx                             ;
                        xor a                           ;
                        scf                             ;
                        ret                             ;

Evals                   inc de                          ;
Eval                    xor a                           ;
                        ld (Flag),A                     ;
                        push bc                         ;
                        push ix                         ;
                        call LECD0                      ;
LEC7A                   call Skip                       ;
                        cp $2B                          ;
                        jr z LEC8F                      ;

                        cp $2D                          ;
                        jr z LEC8F                      ;

                        cp $26                          ;
                        jr z LEC8F                      ;

                        cp $21                          ;
                        jr z LEC8F                      ;

                        jr LECBC                        ;

LEC8F                   push af                         ;
                        push hl                         ;
                        inc de                          ;
                        call LECD0                      ;
                        pop bc                          ;
                        pop af                          ;
                        cp $2D                          ;
                        jr nz LECA2                     ;

                        ld a,c                          ;
                        sub l                           ;
                        ld l,a                          ;
                        ld a,b                          ;
                        sbc a,h                         ;
                        jr LECB9                        ;

LECA2                   cp $2B                          ;
                        jr nz LECA9                     ;

                        add hl,bc                       ;
                        jr LEC7A                        ;

LECA9                   cp $26                          ;
                        jr nz LECB4                     ;

                        ld a,c                          ;
                        and l                           ;
                        ld l,a                          ;
                        ld a,b                          ;
                        and h                           ;
                        jr LECB9                        ;

LECB4                   ld a,c                          ;
                        or l                            ;
                        ld l,a                          ;
                        ld a,b                          ;
                        or h                            ;
LECB9                   ld h,a                          ;
                        jr LEC7A                        ;

LECBC                   pop ix                          ;
                        pop bc                          ;
                        ld a,(Flag)                     ;
                        add a,a                         ;
                        ret nc                          ;

                        ld a,(LastCmdIndexOrAssFlags)   ;
                        bit 1,a                         ;
                        ld hl,$0000                     ;
                        ret z                           ;

                        jp Error9                       ;

LECD0                   call Skip                       ;
                        call Read                       ;
                        ret nc                          ;

                        ld a,(de)                       ;
                        inc de                          ;
                        cp $22                          ;
                        jr z LECF9                      ;

                        ld hl,(LineLength)              ;
                        cp $24                          ;
                        ret z                           ;

                        dec de                          ;
                        call DeclareLabel               ;
                        or a                            ;
                        ld l,(ix+14)                    ;
                        ld h,(ix+15)                    ;
                        bit 7,(ix)                      ;
                        ret nz                          ;

                        ld a,$FF                        ;
                        ld (Flag),A                     ;
                        ret                             ;

LECF9                   ld a,(de)                       ;
                        inc de                          ;
                        ld l,a                          ;
                        or a                            ;
                        ret nz                          ;

                        ld l,$20                        ;
                        dec de                          ;
                        ret                             ;

Evalbs                  inc de                          ;
Evalb                   push hl                         ;
                        call Eval                       ;
                        ld a,l                          ;
                        pop hl                          ;
                        ret                             ;

; Turn the flow warnings off, this horrible code plants "LD HL,NNNN" instructions to skip over the following "LD B,NN"
; instructions. Typical bloody Neil, saving 9 bytes instead of using JR's.

zpush                   zoWarnFlow                      ;
zoWarnFlow=false

Error0                  ld b,$30                        ;
                        db $21                          ;
Error1                  ld b,$31                        ;
                        db $21                          ;
Error2                  ld b,$32                        ;
                        db $21                          ;
Error3                  ld b,$33                        ;
                        db $21                          ;
Error4                  ld b,$34                        ;
                        db $21                          ;
Error5                  ld b,$35                        ;
                        db $21                          ;
Error6                  ld b,$36                        ;
                        db $21                          ;
Error7                  ld b,$37                        ;
                        db $21                          ;
Error9                  ld b,$39                        ;
                        db $21                          ;
Error8                  ld b,$38                        ;

zoWarnFlow=zpop

                        call PrintStringZ               ;
                        db "Error ",0                   ;

                        ld a,b                          ;
                        call PrintChar                  ;

                        ld a,cCR                        ;
                        call PrintChar                  ;

                        exx                             ;
                        ld hl,(SrcLinePointer)          ;
                        ld de,Buffer                    ;
                        ld c,$00                        ;
                        call DeTok_Lp                   ;
                        call LE37C                      ;
                        exx                             ;

                        ld hl,LastCmdIndexOrAssFlags    ;
                        res 0,(hl)                      ;
                        ld hl,(LineLength)              ;
                        ld (AssPC),hl                   ;
                        ld hl,(EntryStackBase)          ;
                        dec hl                          ;
                        dec hl                          ;
                        ld sp,hl                        ;
                        ccf                             ;
                        ret nc                          ;

LED5D                   call SkipSpc                    ;
                        jp z LEBD0                      ;

                        inc de                          ;
                        jr LED5D                        ;

CmdQuit                 ld a,2                          ; Stream 2
                        call ROM_ChanOpen               ; Open

                        ld hl,(Cur_pScreen)             ;
                        call CLS                        ;
                        call PC_StoreCurAndExit         ;
                        ld sp,(EntryStackBase)          ;
                        pop af                          ;
                        pop bc                          ;
                        pop de                          ;
                        pop hl                          ;
                        ex af,af'                       ;
                        exx                             ;
                        pop af                          ;
                        pop bc                          ;
                        pop de                          ;
                        pop hl                          ;
                        ret                             ;

CmdShowStart            call PrintStringZ               ;
                        db "Start of source = ",0       ;

                        ld hl,(SourceStartAddr)         ;
                        call PrintDecimal               ;

                        call PrintStringZ               ;
                        db cCR,"Length          = ",0   ;

                        ld hl,(SourceEndAddr)           ;
                        ld de,(SourceStartAddr)         ;
                        or a                            ;
                        sbc hl,de                       ;
                        inc hl                          ;
                        call PrintDecimal               ;

                        call PrintStringZ               ;
                        db cCR,0                        ;

                        or a                            ;
                        ret                             ;

; Monitor command table

AssCmdTable             db "A",$08                      ;
                        db "D",$08                      ;
                        db "F",$08                      ;
                        db "I",$08                      ;
                        db "L",$08                      ;
                        db "M",$08                      ;
                        db "N",$08                      ;
                        db "O",$08                      ;
                        db "P",$08                      ;
                        db "Q",$08                      ;
                        db "R",$08                      ;
                        db "S",$08                      ;
                        db "T",$08                      ;
                        db "What is the meaning of life ?",$08;
                        db "X",$08                      ;
                        db $0C                          ;

HiddenCommandIndex      equ 14                          ; Index (from 0) +1 into the command table for "What..."

; Command table
; A
AddCmdOffsets           db 1                            ; Number of default parameters
                        dw $000E                        ; Default parameter value
                        dw CmdAssemble                  ; Handler code
; D
                        db 2                            ;
                        dw $FFFF,$0000                  ;
                        dw CmdDelete                    ;
; F
                        db 3                            ;
                        dw $0000,$FFFF,$000E            ;
                        dw CmdFind                      ;
; I
                        db 2                            ;
                        dw $000A,$000A                  ;
                        dw CmdInsert                    ;
; L
                        db 3                            ;
                        dw $0000,$FFFF,$000E            ;
                        dw CmdList                      ;
; M
                        db 0                            ;
                        dw CmdMonitor                   ;
; N
                        db 1                            ;
                        dw $8000                        ;
                        dw CmdNew                       ;
; O
                        db 1                            ;
                        dw $8000                        ;
                        dw CmdOld                       ;
; P
                        db 1                            ;
                        dw $0000                        ;
                        dw CmdPrinter                   ;
; Q
                        db 0                            ;
                        dw CmdQuit                      ;
; R
                        db 3                            ;
                        dw $000A,$000A,$0000            ;
                        dw CmdRenumber                  ;
; S
                        db 1                            ;
                        dw $000E                        ;
                        dw CmdSymbol                    ;
; T
                        db 0                            ;
                        dw CmdShowStart                 ;
; What (etc)
                        db 0                            ;
                        dw CmdShowHidden                ;
; X
                        db 0                            ;
                        dw CmdExecute                   ;

; Token $80
; Tokens are terminated by either $08 or $0A, the difference signals what is allowed to follow them.
; I think $08 means this must be followed by something else? Gawd knows.

TokenDefnTable          db "A",$0A                      ;

                        db "ADC ",$08                   ;

                        db "ADD ",$08                   ;

                        db "AF'",$0A                    ;

                        db "AF",$0A                     ;

                        db "AND ",$08                   ;

                        db "B",$0A                      ;

                        db "BC",$0A                     ;

; Token $88

                        db "BIT ",$08                   ;

                        db "C",$0A                      ;

                        db "CALL ",$08                  ;

                        db "CCF",$0A                    ;

                        db "CP ",$08                    ;

                        db "CPD",$0A                    ;

                        db "CPDR",$0A                   ;

                        db "CPI",$0A                    ;

; Token $90

                        db "CPIR",$0A                   ;

                        db "CPL",$0A                    ;

                        db "D",$0A                      ;

                        db "DAA",$0A                    ;

                        db "DE",$0A                     ;

                        db "DEC ",$08                   ;

                        db "DEFB ",$08                  ;

                        db "DEFM ",$08                  ;

; Token $98

                        db "DEFS ",$08                  ;

                        db "DEFW ",$08                  ;

                        db "DI",$0A                     ;

                        db "DISP ",$08                  ;

                        db "DJNZ ",$08                  ;

                        db "E",$0A                      ;

                        db "EI",$0A                     ;

                        db "ENT",$0A                    ;

; Token $A0

                        db "EQU ",$08                   ;

                        db "EX ",$08                    ;

                        db "EXX",$0A                    ;

                        db "H",$0A                      ;

                        db "HALT",$0A                   ;

                        db "HL",$0A                     ;

                        db "I",$0A                      ;

                        db "IM ",$08                    ;

; Token $A8

                        db "IN ",$08                    ;

                        db "INC ",$08                   ;

                        db "IND",$0A                    ;

                        db "INDR",$0A                   ;

                        db "INI",$0A                    ;

                        db "INIR",$0A                   ;

                        db "IX",$0A                     ;

                        db "IY",$0A                     ;

; Token $B0

                        db "JP ",$08                    ;

                        db "JR ",$08                    ;

                        db "L",$0A                      ;

                        db "LD ",$08                    ;

                        db "LDD",$0A                    ;

                        db "LDDR",$0A                   ;

                        db "LDI",$0A                    ;

                        db "LDIR",$0A                   ;

; Token $B8

                        db "M",$0A                      ;

                        db "NC",$0A                     ;

                        db "NEG",$0A                    ;

                        db "NOP",$0A                    ;

                        db "NV",$0A                     ;

                        db "NZ",$0A                     ;

                        db "OR ",$08                    ;

                        db "ORG ",$08                   ;

; Token $C0

                        db "OTDR",$0A                   ;

                        db "OTIR",$0A                   ;

                        db "OUT ",$08                   ;

                        db "OUTD",$0A                   ;

                        db "OUTI",$0A                   ;

                        db "P",$0A                      ;

                        db "PE",$0A                     ;

                        db "PO",$0A                     ;

; Token $C8

                        db "POP ",$08                   ;

                        db "PUSH ",$08                  ;

                        db "R",$0A                      ;

                        db "RES ",$08                   ;

                        db "RET",$0A                    ;

                        db "RETI",$0A                   ;

                        db "RETN",$0A                   ;

                        db "RL ",$08                    ;

; Token $D0

                        db "RLA",$0A                    ;

                        db "RLC ",$08                   ;

                        db "RLCA",$0A                   ;

                        db "RLD",$0A                    ;

                        db "RR ",$08                    ;

                        db "RRA",$0A                    ;

                        db "RRC ",$08                   ;

                        db "RRCA",$0A                   ;

; Token $D8

                        db "RRD",$0A                    ;

                        db "RST ",$08                   ;

                        db "SBC ",$08                   ;

                        db "SCF",$0A                    ;

                        db "SET ",$08                   ;

                        db "SLA ",$08                   ;

                        db "SP",$0A                     ;

                        db "SRA ",$08                   ;

; Token $E0

                        db "SRL ",$08                   ;

                        db "SUB ",$08                   ;

                        db "V",$0A                      ;

                        db "XOR ",$08                   ;

                        db "Z",$0A                      ;

                        db $0C                          ;


; Format is a data byte followed by the handler address for each token.
; Token $80
TokenHandlerTable       db $00                          ;
                        dw Error8                       ;
                        db $88                          ;
                        dw dwADD_Etc                    ;
                        db $80                          ;
                        dw dwADD_Etc                    ;
                        db $00                          ;
                        dw Error8                       ;
                        db $00                          ;
                        dw Error8                       ;
                        db $A0                          ;
                        dw dwAND_Etc                    ;
                        db $00                          ;
                        dw Error8                       ;
                        db $00                          ;
                        dw Error8                       ;

; Token $88
                        db $40                          ;
                        dw dwBIT_Etc                    ;
                        db $00                          ;
                        dw Error8                       ;
                        db $C4                          ;
                        dw dwCALL_Etc                   ;
                        db $3F                          ;
                        dw dwFixedByte_C                ;
                        db $B8                          ;
                        dw dwAND_Etc                    ;
                        db $A9                          ;
                        dw dwLDI_Etc                    ;
                        db $B9                          ;
                        dw dwLDI_Etc                    ;
                        db $A1                          ;
                        dw dwLDI_Etc                    ;

; Token $90
                        db $B1                          ;
                        dw dwLDI_Etc                    ;
                        db $2F                          ;
                        dw dwFixedByte_C                ;
                        db $00                          ;
                        dw Error8                       ;
                        db $27                          ;
                        dw dwFixedByte_C                ;
                        db $00                          ;
                        dw Error8                       ;
                        db $05                          ;
                        dw dwDEC_Etc                    ;
                        db $00                          ;
                        dw dwDEFB                       ;
                        db $00                          ;
                        dw dwDEFM                       ;

; Token $98
                        db $00                          ;
                        dw dwDEFS                       ;
                        db $00                          ;
                        dw dwDEFW                       ;
                        db $F3                          ;
                        dw dwFixedByte_C                ;
                        db $00                          ;
                        dw dwDISP                       ;
                        db $10                          ;
                        dw dwDJNZ                       ;
                        db $00                          ;
                        dw Error8                       ;
                        db $FB                          ;
                        dw dwFixedByte_C                ;
                        db $00                          ;
                        dw dwENT                        ;

; Token $A0
                        db $00                          ;
                        dw Error8                       ;
                        db $00                          ;
                        dw dwEX                         ;
                        db $D9                          ;
                        dw dwFixedByte_C                ;
                        db $00                          ;
                        dw Error8                       ;
                        db $76                          ;
                        dw dwFixedByte_C                ;
                        db $00                          ;
                        dw Error8                       ;
                        db $00                          ;
                        dw Error8                       ;
                        db $00                          ;
                        dw dwIM                         ;

; Token $A8
                        db $40                          ;
                        dw dwIN_Etc                     ;
                        db $04                          ;
                        dw dwDEC_Etc                    ;
                        db $AA                          ;
                        dw dwLDI_Etc                    ;
                        db $BA                          ;
                        dw dwLDI_Etc                    ;
                        db $A2                          ;
                        dw dwLDI_Etc                    ;
                        db $B2                          ;
                        dw dwLDI_Etc                    ;
                        db $00                          ;
                        dw Error8                       ;
                        db $00                          ;
                        dw dwIY_AndEnableHiddenMsg      ;

; Token $B0
                        db $C2                          ;
                        dw dwJP_Etc                     ;
                        db $20                          ;
                        dw dwJR_Etc                     ;
                        db $00                          ;
                        dw Error8                       ;
                        db $40                          ;
                        dw dwLD                         ;
                        db $A8                          ;
                        dw dwLDI_Etc                    ;
                        db $B8                          ;
                        dw dwLDI_Etc                    ;
                        db $A0                          ;
                        dw dwLDI_Etc                    ;
                        db $B0                          ;
                        dw dwLDI_Etc                    ;

; Token $B8
                        db $00                          ;
                        dw Error8                       ;
                        db $00                          ;
                        dw Error8                       ;
                        db $44                          ;
                        dw dwLDI_Etc                    ;
                        db $00                          ;
                        dw dwFixedByte_C                ;
                        db $00                          ;
                        dw Error8                       ;
                        db $00                          ;
                        dw Error8                       ;
                        db $B0                          ;
                        dw dwAND_Etc                    ;
                        db $00                          ;
                        dw dwORG                        ;

; Token $C0
                        db $BB                          ;
                        dw dwLDI_Etc                    ;
                        db $B3                          ;
                        dw dwLDI_Etc                    ;
                        db $41                          ;
                        dw dwOUT_Etc                    ;
                        db $AB                          ;
                        dw dwLDI_Etc                    ;
                        db $A3                          ;
                        dw dwLDI_Etc                    ;
                        db $00                          ;
                        dw Error8                       ;
                        db $00                          ;
                        dw Error8                       ;
                        db $00                          ;
                        dw Error8                       ;

; Token $C8
                        db $C1                          ;
                        dw dwPUSH                       ;
                        db $C5                          ;
                        dw dwPUSH                       ;
                        db $00                          ;
                        dw Error8                       ;
                        db $80                          ;
                        dw dwBIT_Etc                    ;
                        db $C0                          ;
                        dw dwRET_Etc                    ;
                        db $4D                          ;
                        dw dwLDI_Etc                    ;
                        db $45                          ;
                        dw dwLDI_Etc                    ;
                        db $10                          ;
                        dw dwRL_Etc                     ;

; Token $D0
                        db $17                          ;
                        dw dwFixedByte_C                ;
                        db $00                          ;
                        dw dwRL_Etc                     ;
                        db $07                          ;
                        dw dwFixedByte_C                ;
                        db $6F                          ;
                        dw dwLDI_Etc                    ;
                        db $18                          ;
                        dw dwRL_Etc                     ;
                        db $1F                          ;
                        dw dwFixedByte_C                ;
                        db $08                          ;
                        dw dwRL_Etc                     ;
                        db $0F                          ;
                        dw dwFixedByte_C                ;

; Token $D8
                        db $67                          ;
                        dw dwLDI_Etc                    ;
                        db $C7                          ;
                        dw dwRST                        ;
                        db $98                          ;
                        dw dwADD_Etc                    ;
                        db $37                          ;
                        dw dwFixedByte_C                ;
                        db $C0                          ;
                        dw dwBIT_Etc                    ;
                        db $20                          ;
                        dw dwRL_Etc                     ;
                        db $00                          ;
                        dw Error8                       ;
                        db $28                          ;
                        dw dwRL_Etc                     ;

; Token $E0
                        db $38                          ;
                        dw dwRL_Etc                     ;
                        db $90                          ;
                        dw dwAND_Etc                    ;
                        db $00                          ;
                        dw Error8                       ;
                        db $A8                          ;
                        dw dwAND_Etc                    ;
                        db $00                          ;
                        dw Error8                       ;

dwIY_AndEnableHiddenMsg ld hl,EncryptedCopyrightMsg     ;
                        ld (HiddenMsgBase),hl           ;
                        jp Error8                       ;

CmdPrinter              ld a,(AssParam1)                ;
                        ld (PrinterFlag),A              ;
                        ld a,cCR                        ;
                        call DrivePrinterA              ;
                        or a                            ;
                        ret                             ;

; Encoded "Copyright Neil..." (etc)
EncryptedCopyrightMsg   db $02,$08,$4B,$60,$2F,$46,$2F,$63 ;
                        db $60,$60,$64,$2F,$63,$66,$64,$6A ;
                        db $2F,$4B,$6A,$6A,$7F,$2F,$7B,$67 ;
                        db $60,$7A,$68,$67,$7B,$2F,$30,$02 ;
                        db $08,$02,$08,$55,$6A,$7A,$7C,$02 ;
                        db $27,$6C,$26,$2F,$3C,$20,$2F,$3D ;
                        db $20,$37,$3C,$02,$08,$5C,$21,$4D ;
                        db $7D,$6E,$7B,$7B,$6A,$63,$2F,$29 ;
                        db $2F,$41,$21,$42,$60,$7B,$7B,$6A ;
                        db $7D,$7C,$67,$6A,$6E,$6B,$02,$08 ;
                        db $4E,$63,$63,$2F,$7D,$66,$68,$67 ;
                        db $7B,$7C,$2F,$7D,$6A,$7C,$6A,$7D ;
                        db $79,$6A,$6B,$02,$00          ;

; Monitor stuff below this, written by Crem... Spot my much simpler style ;)

CmdMonitor              call PrintStringZ               ;
                        db "Monitor     ",cCR,0         ;

MonitorLp               call InputLine                  ;
                        ld a,(de)                       ;
                        inc de                          ;
                        cp 'Z'                          ;
                        jr z MonZeus                    ;

                        cp 'C'                          ;
                        jp z MonCopy                    ;

                        cp 'T'                          ;
                        jp z MonTabulate                ;

                        cp 'M'                          ;
                        jp z MonModify                  ;

                        cp 'E'                          ;
                        jp z MonExecute                 ;

                        cp 'K'                          ;
                        jp z MonSetColours              ;

                        cp 'O'                          ;
                        jp z MonOutput                  ;

                        cp 'I'                          ;
                        jp z MonInput                   ;

                        cp 'A'                          ;
                        jp z MonCmdA                    ;

                        cp 'S'                          ;
                        jp z MonSetTabs                 ;

                        jr MonitorLp                    ;

MonSetTabs              ld hl,TabStopData               ;
                        ld bc,31                        ;
                        ex de,hl                        ;
                        ldir                            ;
                        jp MonitorLp                    ;

                        ld hl,TabStopBuffer             ;

MonZeus                 call PrintStringZ               ;
                        db "Zeus",cCR,0                 ;

                        ret                             ;

MonModify               call MonSkipSpc                 ;
                        or a                            ;
                        jr nz MonModifyNoParam          ;

MonModifySlash          call MonReadNumHL               ;
                        ld a,b                          ;
                        or a                            ;
                        jr nz MonModifyAddrError        ;

                        ld (AssParam1),hl               ;
MonModifyLp             ld a,(de)                       ;
                        inc de                          ;
                        cp 0                            ;
                        jr z MonModifyEOL               ;

                        cp '/'                          ;
                        jr z MonModifySlash             ;

                        cp '.'                          ;
                        jr z MonitorLp                  ;

                        cp '"'                          ;
                        jr z MonModifyChar              ;

                        cp ' '                          ;
                        jr z MonModifyLp                ;

                        dec de                          ;
                        call MonReadNumA                ;
                        ex af,af'                       ;
                        ld a,b                          ;
                        or a                            ;
                        jr nz MonModifyNumError         ;

                        ex af,af'                       ;
MonModifyWriteByte      ld (hl),a                       ;
                        inc hl                          ;
                        jr MonModifyLp                  ;

MonModifyNumError       call NumError                   ;

MonModifyEOL            call PrintHexWord               ;

                        ld a,(hl)                       ;
                        call PrintHexByte               ;

                        call PrintStringZ               ;
                        db cLEFT,cLEFT,cLEFT,0          ;

                        call InputLine                  ;
                        jr MonModifySlash               ;

MonModifyChar           ld a,(de)                       ;
                        inc de                          ;
                        jr MonModifyWriteByte           ;

AddrError               call PrintStringZ               ;
                        db "Incorrect Address",cCR,0    ;

                        ret                             ;

MonModifyAddrError      call AddrError                  ;
                        ld hl,(AssParam1)               ;
                        jr MonModifyEOL                 ;

MonModifyNoParam        ld hl,(AssParam1)               ;
                        jr MonModifyLp                  ;

NumError                call PrintStringZ               ;
                        db "Incorrect Number",cCR,0     ;
                        ret                             ;

MonReadNumHL            ld hl,0                         ;
                        call LF2A5                      ;
                        ld a,b                          ;
                        or a                            ;
                        ret nz                          ;

                        ld c,$03                        ;
LF297                   call LF2A5                      ;
                        ld a,b                          ;
                        or a                            ;
                        jr nz LF2A2                     ;

                        dec c                           ;
                        jr nz LF297                     ;

                        ret                             ;

LF2A2                   ld b,$00                        ;
                        ret                             ;

LF2A5                   ld a,(de)                       ;
                        inc de                          ;
                        cp '0'                          ;
                        jr c LF2C6                      ;

                        cp '9'+1                        ;
                        jr nc LF2BA                     ;

                        sub '0'                         ;

LF2B1                   add hl,hl                       ;
                        add hl,hl                       ;
                        add hl,hl                       ;
                        add hl,hl                       ;

                        or l                            ;
                        ld l,a                          ;

                        ld b,0                          ;
                        ret                             ;

LF2BA                   cp 'A'                          ;
                        jr c LF2C6                      ;

                        cp 'F'+1                        ;
                        jr nc LF2C6                     ;

                        sub '7'                         ;
                        jr LF2B1                        ;

LF2C6                   dec de                          ;
                        ld b,$01                        ;
                        ret                             ;

MonReadNumA             push hl                         ;
                        call MonReadNumHL               ;
                        ld a,l                          ;
                        pop hl                          ;
                        ret                             ;

                        ld a,h                          ;
                        call PrintHexByte               ;
                        ld a,cLEFT                      ;
                        call PrintChar                  ;
                        ld a,l                          ;
                        call PrintHexByte               ;
                        ret                             ;

PrintHexByte            push af                         ;
                        ld c,a                          ;

                        rra                             ;
                        rra                             ;
                        rra                             ;
                        rra                             ;

                        call PrintHexNibble             ;

                        ld a,c                          ;
                        call PrintHexNibble             ;

                        ld a,' '                        ;
                        call PrintChar                  ;

                        pop af                          ;
                        ret                             ;

PrintHexNibble          and $0F                         ;
                        add a,'0'                       ;
                        cp '9'+1                        ;
                        jp c PrintChar                  ;

                        add a,'A'-('9'+1)               ;
                        jp PrintChar                    ;

MonCopy                 call MonSkipSpc                 ;
                        or a                            ;
                        jr nz MonCopyFail               ;

                        call MonReadNumHL               ;
                        ld a,b                          ;
                        or a                            ;
                        jr nz MonCopyFail               ;

                        ld (AssParam1),hl               ;
                        call MonSkipSpc                 ;
                        or a                            ;
                        jr nz MonCopyFail               ;

                        call MonReadNumHL               ;
                        ld a,b                          ;
                        or a                            ;
                        jp nz MonNumericError           ;

                        ld (AssParam2),hl               ;
                        call MonSkipSpc                 ;
                        or a                            ;
                        jp nz MonCopyFail               ;

                        call MonReadNumHL               ;
                        ld a,b                          ;
                        or a                            ;
                        jp nz MonNumericError           ;

                        ld (AssParam3),hl               ;
                        ld hl,(AssParam1)               ;
                        ld de,(AssParam2)               ;
                        ld bc,(AssParam3)               ;
                        ldir                            ;
                        jp MonitorLp                    ;

MonCopyFail             call PrintStringZ               ;
                        db cCR,"Copy must have 3 parameters ",cCR,0;
                        jp MonitorLp                    ;

MonSkipSpc              ld a,(de)                       ;
                        cp 0                            ;
                        jr z MonSkipSpcEOL              ;

                        cp ' '                          ;
                        jr nz MonSkipSpcOK              ;

                        inc de                          ;
                        jr MonSkipSpc                   ;

MonSkipSpcEOL           ld a,1                          ;
                        ret                             ;

MonSkipSpcOK            ld a,0                          ;
                        ret                             ;

MonExecute              call MonSkipSpc                 ;
                        or a                            ;
                        jr nz MinExecuteFail            ;

                        call MonReadNumHL               ;
                        ld a,b                          ;
                        or a                            ;
                        jr nz MinExecuteFail            ;

                        ld (AssParam1),hl               ;
                        ld de,MonitorLp                 ;
                        push de                         ;
                        jp (hl)                         ;

MinExecuteFail          call PrintStringZ               ;
                        db "Execute must have a parameter",cCR,0;
                        jp MonitorLp                    ;

MonTabulate             ld a,$08                        ;
                        ld (AssParam2),A                ;
                        call MonSkipSpc                 ;
                        or a                            ;
                        jr nz MonTab_Lp                 ;

                        call MonReadNumHL               ;
                        ld a,b                          ;
                        or a                            ;
                        jp nz MonTab_Lp                 ;

                        ld (AssParam1),hl               ;
                        call MonSkipSpc                 ;
                        or a                            ;
                        jr nz MonTab_Lp                 ;

                        call MonReadNumA                ;
                        ex af,af'                       ;
                        ld a,b                          ;
                        or a                            ;
                        jr nz MonTab_Lp                 ;

                        ex af,af'                       ;
                        ld (AssParam2),A                ;
MonTab_Lp               ld hl,(AssParam1)               ;
MonTab_Next             ld a,(AssParam2)                ;
                        ld c,a                          ;

MonTab_Lp1              push bc                         ;

                        ld a,cCR                        ;
                        call PrintChar                  ;

                        call PrintHexWord               ;

                        ld b,8                          ;
MonTab_Lp2              push bc                         ;
                        ld a,(hl)                       ;
                        inc hl                          ;
                        call PrintHexByte               ;
                        pop bc                          ;
                        djnz MonTab_Lp2                 ;

                        pop bc                          ;
                        dec c                           ;
                        jr nz MonTab_Lp1                ;

                        call InputChar                  ;
                        cp cCR                          ;
                        jr z MonTab_Next                ;

                        ld a,cCR                        ;
                        call PrintChar                  ;
                        jp MonitorLp                    ;

MonNumericError         call NumError                   ;
                        jp MonitorLp                    ;

MonSetColours           call MonSkipSpc                 ;
                        or a                            ;
                        jp nz MonitorLp                 ;

                        call MonReadNumA                ;
                        ex af,af'                       ;
                        ld a,b                          ;
                        or a                            ;
                        jp nz MonitorLp                 ;

                        ex af,af'                       ;
                        ld (CLS_cAttr),A                ;
                        call MonSkipSpc                 ;
                        or a                            ;
                        jp nz MSC_DoIt                  ;

                        call MonReadNumA                ;
                        ex af,af'                       ;
                        ld a,b                          ;
                        or a                            ;
                        jp nz MonitorLp                 ;

                        ex af,af'                       ;
                        ld (BdrCol),A                   ;
MSC_DoIt                call CLS_Attrs                  ;
                        jp MonitorLp                    ;

MonOutput               call MonSkipSpc                 ;
                        or a                            ;
                        jp nz MonitorLp                 ;

                        call MonReadNumA                ;
                        ld c,a                          ;
                        ld a,b                          ;
                        or a                            ;
                        jp nz MonitorLp                 ;

                        push bc                         ;
                        call MonSkipSpc                 ;
                        call MonReadNumA                ;
                        ld c,a                          ;
                        ld a,b                          ;
                        or a                            ;
                        jp nz MonitorLp                 ;

                        ld a,c                          ;
                        pop bc                          ;
                        out (c),a                       ;
                        jp MonitorLp                    ;

MonInput                call MonSkipSpc                 ;
                        or a                            ;
                        jp nz MonitorLp                 ;

                        call MonReadNumHL               ;
                        ld a,b                          ;
                        or a                            ;
                        jp nz MonitorLp                 ;

                        ld b,h                          ;
                        ld c,l                          ;
                        in a,(C)                        ;
                        call PrintHexByte               ;
                        ld a,cCR                        ;
                        call PrintChar                  ;
                        jp MonitorLp                    ;

MonCmdA                 call Read                       ;
                        jp c MonitorLp                  ;

                        call PrintStringZ               ;
                        db cCR,"Hex = ",0               ;

                        call PrintHexWord               ;

                        call PrintStringZ               ;
                        db "Dec = ",0                   ;

                        call PrintDecNZ                 ;

                        ld a,cCR                        ;
                        call PrintChar                  ;

                        jp MonitorLp                    ;

CmdShowHidden           ld hl,sHiddenMsg2               ;
HiddenMsgBase           equ *-2                         ;
PrintHiddenHL           ld a,(hl)                       ;
                        or a                            ;
                        ret z                           ;

                        xor $0F                         ;
                        call PrintChar                  ;
                        inc hl                          ;
                        jr PrintHiddenHL                ;

; Encoded "42 "+Cret
sHiddenMsg2             db $02,$3B,$3D,$2F,$02,$00      ;

Cur_pScreen             dw VDU_Base                     ;

                        ; These seem to be involved in reading the keyboard

KBD_UnkWordVar2         dw $007D                        ;
KBD_UnkWordVar3         dw $007D                        ; This needs to be defined, never written
KBD_UnkWordVar4         dw $0018                        ; This needs to be defined, never written

KBD_UnkByteVar1         db $0D                          ;
NoSourceFlag            db $FF                          ;
KBD_UnkByteVar3         db $00                          ;
KBD_UnkByteVar4         db $38                          ;
KBD_UnkByteVar5         db $08                          ;
PrinterFlag             db $00                          ;

CLS                     ld hl,VDU_Base                  ;
                        ld de,VDU_Base+1                ;
                        ld bc,lPixels                   ;
                        ld (Cur_pScreen),hl             ;
                        ld (hl),$00                     ;
                        ldir                            ;

                        call PC_StoreCurAndExit         ;

CLS_Attrs               ld hl,ATTR_Base                 ;
                        ld de,ATTR_Base+1               ;
                        ld (hl),$0F                     ;
CLS_cAttr               equ *-1                         ;
                        ld bc,lAttrs                    ;
                        ldir                            ;
                        ld a,$01                        ;
BdrCol                  equ *-1                         ;
                        out ($FE),a                     ;
                        ret                             ;

PC_StoreCurAndExit      ld hl,(Cur_pScreen)             ;
PC_ShowCursor           push hl                         ;
                        push af                         ;
                        ld a,7                          ;
                        add a,h                         ;
                        ld h,a                          ;
                        ld a,(hl)                       ;
                        xor $7E                         ;
                        ld (hl),a                       ;
                        ld b,$00                        ;
                        pop af                          ;
                        pop hl                          ;
                        ret                             ;

PrintChar               push af                         ;
                        push hl                         ;
                        ld l,a                          ;
                        ld a,(PrinterFlag)              ;
                        or a                            ;
                        jr z PC_NotPrinter              ;

                        ld a,l                          ;
                        push de                         ;
                        push bc                         ;
                        cp ' '                          ;
                        jr nc PC_NotCtrl                ;

                        cp cCR                          ;
                        jr nz PC_NotCret                ;

PC_NotCtrl              call DrivePrinterA              ;

PC_NotCret              pop bc                          ;
                        pop de                          ;
PC_NotPrinter           pop hl                          ;
                        pop af                          ;

PrintCharSaveReg        push hl                         ;
                        push de                         ;
                        push bc                         ;
                        push af                         ;
                        call PrintCharInner             ;
                        pop af                          ;
                        pop bc                          ;
                        pop de                          ;
                        pop hl                          ;
                        ret                             ;

PrintCharInner          ld hl,(Cur_pScreen)             ;
                        call PC_ShowCursor              ;
                        cp ' '                          ;
                        jp c PC_Ctrl                    ;

                        cp $80                          ;
                        jp nc vPC_ShowCursor            ;

                        call PC_DrawChar                ;

PC_MoveRight            ld hl,(Cur_pScreen)             ;
                        inc l                           ;
                        ld a,l                          ;
                        or a                            ;
                        jp z PC_MR_Wrap                 ;

                        ld (Cur_pScreen),hl             ;
                        jp PC_ShowCursor                ;

PC_MR_Wrap              ld a,h                          ;
                        add a,$08                       ;
                        cp $58                          ;
                        jr z PC_MR_EOS                  ;

                        ld h,a                          ;
                        jp PC_ML_X                      ;

PC_MR_EOS               call ScrollUp                   ;
                        ld hl,$50E0                     ;
                        ld (Cur_pScreen),hl             ;
                        jp PC_ShowCursor                ;

ClearBottomLine         ld hl,$50E0                     ;
ClearCharLine           push hl                         ;
                        ld b,8                          ;
ClearCharLineLp         call ClearPixelLine             ;
                        inc h                           ;
                        djnz ClearCharLineLp            ;

                        pop hl                          ;
                        ld a,l                          ;
                        and $E0                         ;
                        ld l,a                          ;
                        ret                             ;

ClearPixelLine          push bc                         ;
                        ld d,h                          ;
                        ld e,l                          ;
                        push hl                         ;
                        inc de                          ;
                        ld bc,$001F                     ;
                        ld (hl),$00                     ;
                        ldir                            ;
                        pop hl                          ;
                        pop bc                          ;
                        ret                             ;

PC_Ctrl                 cp cRIGHT                       ;
                        jp z PC_MoveRight               ;

                        cp cLEFT                        ;
                        jp z PC_MoveLeft                ;

                        cp cLF                          ;
                        jp z PC_MoveDown                ;

                        cp cUP                          ;
                        jp z PC_MoveUp                  ;

                        cp cDELETE                      ;
                        jp z PC_Delete                  ;

                        cp cCLS                         ;
                        jp z CLS                        ;

                        cp cCR                          ;
                        jp z PC_Cret                    ;

                        cp cINSERT                      ;
                        jp z PC_Insert                  ;

                        cp cDELRIGHT                    ;
                        jp z PC_DeleteRight             ;

                        cp cCLEARLINE                   ;
                        jp z PC_ClearLine               ;

                        cp cTAB                         ;
                        jp z PC_Tab                     ;

vPC_ShowCursor          jp PC_ShowCursor                ;

PC_Cret                 ld a,l                          ;
                        and $E0                         ;
                        ld l,a                          ;
                        jp PC_MoveDown                  ;

PC_MoveLeft             ld a,l                          ;
                        and $1F                         ;
                        jr z PC_ML_AtSOL                ;

                        dec l                           ;
PC_ML_X                 ld (Cur_pScreen),hl             ;
                        jp PC_ShowCursor                ;

PC_ML_AtSOL             ld a,l                          ;
                        sub $20                         ;
                        ld l,a                          ;
                        jr nc PC_ML_Up                  ;

                        ld a,h                          ;
                        sub $08                         ;
                        ld h,a                          ;
                        cp $38                          ;
                        jr nz PC_ML_Up                  ;

                        ld hl,VDU_Base                  ;
PC_ML_Up                ld a,l                          ;
                        or $1F                          ;
                        ld l,a                          ;
                        jr PC_ML_X                      ;

PC_MoveDown             ld a,$20                        ;
                        add a,l                         ;
                        ld l,a                          ;
                        and $E0                         ;
                        jp z PC_MR_Wrap                 ;

                        jp PC_ML_X                      ;

PC_MoveUp               ld a,l                          ;
                        sub $20                         ;
                        ld l,a                          ;
                        jr nc PC_ML_X                   ;

                        ld a,h                          ;
                        sub $08                         ;
                        ld h,a                          ;
                        cp $38                          ;
                        jr nz PC_ML_X                   ;

                        ld h,$40                        ;
                        ld a,l                          ;
                        and $1F                         ;
                        ld l,a                          ;
                        jr PC_ML_X                      ;

PC_Delete               call PC_MoveLeft                ;
                        ld b,7                          ;
PC_D_Lp                 ld (hl),$00                     ;
                        inc h                           ;
                        djnz PC_D_Lp                    ;

                        ld (hl),$7E                     ;
                        inc h                           ;
                        ret                             ;

PC_Tab                  ld hl,TabStopBuffer             ;
                        ld d,$00                        ;
                        ld a,(Cur_pScreen)              ;
                        and $1F                         ;
                        ld e,a                          ;
                        add hl,de                       ;
PC_Tab_Lp               inc hl                          ;
                        inc de                          ;
                        ld a,(hl)                       ;
                        or a                            ;
                        jp z PC_StoreCurAndExit         ;

                        cp $20                          ;
                        jr z PC_Tab_Lp                  ;

                        ld a,(Cur_pScreen)              ;
                        and $E0                         ;
                        or e                            ;
                        ld (Cur_pScreen),A              ;
                        jp PC_StoreCurAndExit           ;

PC_DrawChar             push hl                         ;
                        ld de,(ROM_CharsPtr)            ;
                        ld l,a                          ;
                        ld h,$00                        ;
                        add hl,hl                       ;
                        add hl,hl                       ;
                        add hl,hl                       ;
                        add hl,de                       ;
                        ex de,hl                        ;
                        pop hl                          ;
                        ld b,8                          ;
PC_DC_Lp                ld a,(de)                       ;
                        ld (hl),a                       ;
                        inc de                          ;
                        inc h                           ;
                        djnz PC_DC_Lp                   ;

                        ret                             ;

InputChar               ei                              ;
                        push hl                         ;
                        push bc                         ;
                        xor a                           ;
                        ld (KBD_UnkByteVar3),A          ;
                        ld a,(KBD_UnkByteVar4)          ;
                        ld (KBD_UnkByteVar5),A          ;
InputChar1              ld a,(KBD_UnkByteVar5)          ;
                        dec a                           ;
                        ld (KBD_UnkByteVar5),A          ;
                        jp nz InputChar2                ;

                        ld hl,(Cur_pScreen)             ;
                        call PC_ShowCursor              ;
                        ld a,(KBD_UnkByteVar3)          ;
                        xor $01                         ;
                        ld (KBD_UnkByteVar3),A          ;
                        ld a,(KBD_UnkByteVar4)          ;
                        ld (KBD_UnkByteVar5),A          ;
InputChar2              call Delay                      ;
                        ld a,(ROM_Unk)                  ;
                        or a                            ;
                        jr nz InputChar3                ;

                        xor a                           ;
                        ld (KBD_UnkByteVar1),A          ;
                        jr InputChar6                   ;

InputChar3              ld a,(KBD_UnkByteVar1)          ;
                        ld b,a                          ;
                        ld a,(ROM_LastK)                ;
                        cp b                            ;
                        jr z InputChar4                 ;

                        ld (KBD_UnkByteVar1),A          ;
                        ld hl,(KBD_UnkWordVar3)         ;
                        ld (KBD_UnkWordVar2),hl         ;
                        jr InputChar6                   ;

InputChar4              ld hl,(KBD_UnkWordVar2)         ;
                        dec hl                          ;
                        ld (KBD_UnkWordVar2),hl         ;
                        ld a,h                          ;
                        or l                            ;
                        jr z InputChar5                 ;

                        xor a                           ;
                        jr InputChar6                   ;

InputChar5              ld hl,(KBD_UnkWordVar4)         ;
                        ld (KBD_UnkWordVar2),hl         ;
                        ld a,(KBD_UnkByteVar1)          ;
InputChar6              or a                            ;
                        jr z InputChar1                 ;

                        push af                         ;
                        ld a,(KBD_UnkByteVar3)          ;
                        or a                            ;
                        jr z InputChar7                 ;

                        ld hl,(Cur_pScreen)             ;
                        call PC_ShowCursor              ;
InputChar7              pop af                          ;
                        cp 'A'                          ;
                        jr c InputCharX                 ;

                        cp 'Z'+1                        ;
                        jr nc InputChar8                ;

                        add a,'a'-'A'                   ;
                        jr InputCharX                   ;

InputChar8              cp 'a'                          ;
                        jr c InputCharX                 ;

                        cp 'z'+1                        ;
                        jr nc InputCharX                ;

                        sub 'a'-'A'                     ;
InputCharX              pop bc                          ;
                        pop hl                          ;
                        ret                             ;

InputLine               call InputChar                  ;
                        call PrintCharSaveReg           ;
                        cp cCR                          ;
                        jr nz InputLine                 ;

                        call ReadLine                   ;
                        ld a,(PrinterFlag)              ;
                        or a                            ;
                        jp nz PrintLineFromDE           ;

                        ret                             ;

PC_Insert               ld hl,(Cur_pScreen)             ;
                        ld a,l                          ;
                        or $1F                          ;
                        ld l,a                          ;
                        ld d,h                          ;
                        ld e,l                          ;
                        dec hl                          ;
                        ld a,(Cur_pScreen)              ;
                        cpl                             ;
                        and $1F                         ;
                        jr z PC_InsertClr               ;

                        ld b,8                          ;
PC_Insert_Lp            push hl                         ;
                        push de                         ;
                        push bc                         ;
                        ld b,$00                        ;
                        ld c,a                          ;
                        lddr                            ;
                        pop bc                          ;
                        pop de                          ;
                        pop hl                          ;
                        inc d                           ;
                        inc h                           ;
                        djnz PC_Insert_Lp               ;

PC_InsertClr            ld hl,(Cur_pScreen)             ;
                        ld b,7                          ;
PC_InsertClr_Lp         ld (hl),$00                     ;
                        inc h                           ;
                        djnz PC_InsertClr_Lp            ;

                        ld (hl),$7E                     ;
                        inc h                           ;
                        ret                             ;

PC_DeleteRight          ld hl,(Cur_pScreen)             ;
                        ld d,h                          ;
                        ld e,l                          ;
                        inc hl                          ;
                        ld a,e                          ;
                        cpl                             ;
                        and $1F                         ;
                        jr z PC_InsertClr               ;

                        ld b,8                          ;
PC_DeleteLp             push hl                         ;
                        push de                         ;
                        push bc                         ;
                        ld b,$00                        ;
                        ld c,a                          ;
                        ldir                            ;
                        dec hl                          ;
                        ld (hl),$00                     ;
                        pop bc                          ;
                        pop de                          ;
                        pop hl                          ;
                        inc d                           ;
                        inc h                           ;
                        djnz PC_DeleteLp                ;

                        ld hl,(Cur_pScreen)             ;
                        jp PC_ShowCursor                ;

ReadLine                ld hl,(Cur_pScreen)             ;
                        ld a,l                          ;
                        sub $20                         ;
                        ld l,a                          ;
                        jr nc ReadLine1                 ;

                        ld a,h                          ;
                        sub $08                         ;
                        ld h,a                          ;
                        cp $38                          ;
                        jr nz ReadLine1                 ;

                        ld h,$40                        ;
                        ld a,l                          ;
                        and $1F                         ;
                        ld l,a                          ;
ReadLine1               ld a,l                          ;
                        and $E0                         ;
                        ld l,a                          ;
                        ld de,Buffer                    ;
                        ld b,$20                        ;
ReadLineLp              call FindChar                   ;
                        ld (de),a                       ;
                        inc hl                          ;
                        inc de                          ;
                        djnz ReadLineLp                 ;

                        ld a,$00                        ;
                        ld (de),a                       ;
                        ld de,Buffer                    ;
                        ret                             ;

FindChar                push de                         ;
                        push bc                         ;
                        ld de,$3D00                     ;
                        ld b,$60                        ;
FindCharLp              call CompChar                   ;
                        jr nz FC_Found                  ;

                        ld a,$07                        ;
                        or e                            ;
                        ld e,a                          ;
                        inc de                          ;
                        djnz FindCharLp                 ;

                        ld a,$20                        ;
FC_Found                pop bc                          ;
                        pop de                          ;
                        ret                             ;

CompChar                push hl                         ;
                        push bc                         ;
                        ld b,8                          ;
CompCharLp              ld a,(de)                       ;
                        cp (hl)                         ;
                        jr nz CompCharX                 ;

                        inc h                           ;
                        inc de                          ;
                        djnz CompCharLp                 ;

                        pop bc                          ;
                        ld a,$80                        ;
                        sub b                           ;
                        pop hl                          ;
                        ret                             ;

CompCharX               pop bc                          ;
                        pop hl                          ;
                        xor a                           ;
                        ret                             ;

Delay                   ld c,8                          ;

DelayLp                 ld b,$40                        ;
DelayLp2                nop                             ;
                        djnz DelayLp2                   ;

                        dec c                           ;
                        jr nz DelayLp                   ;

                        ret                             ;

PC_ClearLine            ld hl,(Cur_pScreen)             ;
                        ld a,l                          ;
                        and $E0                         ;
                        ld l,a                          ;
                        call ClearCharLine              ;
                        ld (Cur_pScreen),hl             ;
                        jp PC_ShowCursor                ;

ScrollUp                ld hl,VDU_Base                  ;
                        ld b,8                          ;
ScrollUpLp              push hl                         ;
                        push bc                         ;
                        call ScrollUpInner              ;
                        pop bc                          ;
                        pop hl                          ;
                        inc h                           ;
                        djnz ScrollUpLp                 ;

                        jp ClearBottomLine              ;

ScrollUpInner           ld d,h                          ;
                        ld e,l                          ;
                        ld a,l                          ;
                        add a,$20                       ;
                        ld l,a                          ;
                        ld bc,$00E0                     ;
                        ldir                            ;
                        push de                         ;
                        ld de,$0700                     ;
                        add hl,de                       ;
                        pop de                          ;
                        ld bc,$0020                     ;
                        ldir                            ;
                        ld d,h                          ;
                        ld e,l                          ;
                        ld a,e                          ;
                        sub $20                         ;
                        ld e,a                          ;
                        ld bc,$00E0                     ;
                        ldir                            ;
                        push de                         ;
                        ld de,$0700                     ;
                        add hl,de                       ;
                        pop de                          ;
                        ld bc,$0020                     ;
                        ldir                            ;
                        ld d,h                          ;
                        ld e,l                          ;
                        ld a,e                          ;
                        sub $20                         ;
                        ld e,a                          ;
                        ld bc,$00E0                     ;
                        ldir                            ;
                        ret                             ;

PrintLineFromDE         push de                         ;
                        ld b,$20                        ;
PrintLineLp             ld a,(de)                       ;
                        call DrivePrinterA              ;
                        inc de                          ;
                        djnz PrintLineLp                ;

                        pop de                          ;
                        ld a,cCR                        ;
                        call DrivePrinterA              ;
                        ret                             ;

DrivePrinterA           push hl                         ;
                        push de                         ;
                        push bc                         ;
                        push af                         ;
                        exx                             ;
                        ex af,af'                       ;
                        push hl                         ;
                        push de                         ;
                        push bc                         ;
                        push af                         ;
                        ex af,af'                       ;
                        rst $10                         ;
                        pop af                          ;
                        pop bc                          ;
                        pop de                          ;
                        pop hl                          ;
                        exx                             ;
                        ex af,af'                       ;
                        pop af                          ;
                        pop bc                          ;
                        pop de                          ;
                        pop hl                          ;
                        ret                             ;


TabStopDefn             db $00,$20,$20,$20,$20,$20,$31,$20 ;
                        db $20,$20,$20,$20,$31,$20,$20,$20 ;
                        db $31,$20,$20,$20,$31,$20,$20,$20 ;
                        db $20,$20,$20,$20,$20,$20,$20,$20 ;
                        db $00                          ;

TabStopBuffer           db $00                          ;
TabStopData             db $20,$20,$20,$20,$20,$31,$20,$20 ;
                        db $20,$20,$20,$31,$20,$20,$20,$31 ;
                        db $20,$20,$20,$31,$20,$20,$20,$20 ;
                        db $20,$20,$20,$20,$20,$20,$20,$00 ;

AppLast                 equ *

                        org $FD00                       ;
SourceStartAddr         dw $0000                        ;
SourceEndAddr           dw $0000                        ;
MagicNumber             dw $0000                        ;
LastCmdIndexOrAssFlags  db $00                          ; Dual purpose. Monitor uses it as a command index, assembler as flags
SrcLinePointer          dw $0000                        ;
LFD09                   dw $0000                        ;
LineLength              dw $0000                        ;
Flag                    db $00,$00                      ;
AssPC                   dw $0000                        ;
AssDISP                 dw $0000                        ;
SymbolTableBase         dw $0000                        ;
ExecuteAddr             dw $0000                        ;
EntryStackBase          dw $7FD8                        ;
AssParam1               dw $0000                        ;
AssParam2               dw $0000                        ;
AssParam3               dw $0000                        ;

                        org $FE00                       ;
Buffer                  db $00,$00,$00,$00,$00,$00,$00,$00 ;
                        db $00,$00,$00,$00,$00,$00,$00,$00 ;
                        db $00,$00,$00,$00,$00,$00,$00,$00 ;
                        db $00,$00,$00,$00,$00,$00,$00,$00 ;

LabelBuffer             equ $DFF0                       ;

LineBuffer              equ $FD50                       ;
LFDFF                   equ $FDFF                       ;

; Assembler token values

tA                      equ $80                         ;
tADC                    equ $81                         ;
tADD                    equ $82                         ;
tAFd                    equ $83                         ;
tAF                     equ $84                         ;
tAND                    equ $85                         ;
tB                      equ $86                         ;
tBC                     equ $87                         ;
tBIT                    equ $88                         ;
tC                      equ $89                         ;
tCALL                   equ $8A                         ;
tCCF                    equ $8B                         ;
tCP                     equ $8C                         ;
tCPD                    equ $8D                         ;
tCPDR                   equ $8E                         ;
tCPI                    equ $8F                         ;
tCPIR                   equ $90                         ;
tCPL                    equ $91                         ;
tD                      equ $92                         ;
tDAA                    equ $93                         ;
tDE                     equ $94                         ;
tDEC                    equ $95                         ;
tDEFB                   equ $96                         ;
tDEFM                   equ $97                         ;
tDEFS                   equ $98                         ;
tDEFW                   equ $99                         ;
tDI                     equ $9A                         ;
tDISP                   equ $9B                         ;
tDJNZ                   equ $9C                         ;
tE                      equ $9D                         ;
tEI                     equ $9E                         ;
tENT                    equ $9F                         ;
tEQU                    equ $A0                         ;
tEX                     equ $A1                         ;
tEXX                    equ $A2                         ;
tH                      equ $A3                         ;
tHALT                   equ $A4                         ;
tHL                     equ $A5                         ;
tI                      equ $A6                         ;
tIM                     equ $A7                         ;
tIN                     equ $A8                         ;
tINC                    equ $A9                         ;
tIND                    equ $AA                         ;
tINDR                   equ $AB                         ;
tINI                    equ $AC                         ;
tINIR                   equ $AD                         ;
tIX                     equ $AE                         ;
tIY                     equ $AF                         ;
tJP                     equ $B0                         ;
tJR                     equ $B1                         ;
tL                      equ $B2                         ;
tLD                     equ $B3                         ;
tLDD                    equ $B4                         ;
tLDDR                   equ $B5                         ;
tLDI                    equ $B6                         ;
tLDIR                   equ $B7                         ;
tM                      equ $B8                         ;
tNC                     equ $B9                         ;
tNEG                    equ $BA                         ;
tNOP                    equ $BB                         ;
tNV                     equ $BC                         ;
tNZ                     equ $BD                         ;
tOR                     equ $BE                         ;
tORG                    equ $BF                         ;
tOTDR                   equ $C0                         ;
tOTIR                   equ $C1                         ;
tOUT                    equ $C2                         ;
tOUTD                   equ $C3                         ;
tOUTI                   equ $C4                         ;
tP                      equ $C5                         ;
tPE                     equ $C6                         ;
tPO                     equ $C7                         ;
tPOP                    equ $C8                         ;
tPUSH                   equ $C9                         ;
tR                      equ $CA                         ;
tRES                    equ $CB                         ;
tRET                    equ $CC                         ;
tRETI                   equ $CD                         ;
tRETN                   equ $CE                         ;
tRL                     equ $CF                         ;
tRLA                    equ $D0                         ;
tRLC                    equ $D1                         ;
tRLCA                   equ $D2                         ;
tRLD                    equ $D3                         ;
tRR                     equ $D4                         ;
tRRA                    equ $D5                         ;
tRRC                    equ $D6                         ;
tRRCA                   equ $D7                         ;
tRRD                    equ $D8                         ;
tRST                    equ $D9                         ;
tSBC                    equ $DA                         ;
tSCF                    equ $DB                         ;
tSET                    equ $DC                         ;
tSLA                    equ $DD                         ;
tSP                     equ $DE                         ;
tSRA                    equ $DF                         ;
tSRL                    equ $E0                         ;
tSUB                    equ $E1                         ;
tV                      equ $E2                         ;
tXOR                    equ $E3                         ;
tZ                      equ $E4                         ;

; Uncomment one of these (and supply a binary file) to use Zeus's binary comparison feature to check this code.

; There are no differences between the bytes generated by this source and the original release.
;                        import_comparison "C:\\PROJECTS\\ZeusZ80\\Zeus1\\orig.bin",0
; The only differences between this and the Sinclair release are two bytess (the state of keyboard or summat equally irrelevant)
;                        import_comparison "C:\\PROJECTS\\ZeusZ80\\Zeus1\\sinc.bin",0

; Uncomment one or more of the next lines to generate a tzx file using the loader
                        output_tzx "zeuscode.tzx","Zeus 1","(c) Crem and Mott",Start,AppLast-Start ;  load this with LOAD "" CODE
                        output_tzx "zeus.tzx","Zeus 1","(c) Crem and Mott",Start,AppLast-Start,2,Start ; Load this with LOAD ""


/*

Now, here FYI is the Diana Definition file I used to disassemble Zeus to generate this source.
I've edited the source file this generates a fair bit, so it won't exactly produce what you see up there ^^^
But that's how you ought to use Diana - edit the definition file and disassemble until it becomes easier to edit the source
code she produces. Don't feel you have to do everything in the definition file. Or, indeed, anything.

o0000
fC:\\PROJECTS\\ZeusZ80\\Zeus1\\sinc.bin
LDFF0 LabelBuffer
rE000 FFFF
LE000 Start
LE086 MainCold
LE092 MainWarm
LE0B6 MainLp
LE0CB MainInsertLN
LE0CE MainGetEntryLine
LE124 DT_GetNextWord
LE133 DT_GetNextParam1
LE13F DT_GNP1
LE15B DT_GNP2

LE4B0 IfLetterUpperOnly_NC
LE4C2 IfLetterEitherCase_NC
LE4B7 IfNumber_NC
LE4BE IfIdentChar_NC
LE4D3 IfHexChar_NC
LE4CD IfPfx_NC
LE4F2 Skip2
LE4F3 Skip1
LE4F4 Skip
LE4FF SkipT
LE50A Rdstg
LE50D Rdstg1
LE51B Rdstg2
LE520 Rdstg3
LE524 Rdstg4
LE52C Read
LE557 ReadHex
LE53C Read1
LE55E Read3
LE6B2 C0ish
LE748 SkipComma
LEC6F Evals
LEC70 Eval
LED02 Evalbs
LED03 Evalb
LED0A Error0
LED0D Error1
LED10 Error2
LED13 Error3
LED16 Error4
LED19 Error5
LED1C Error6
LED1F Error7
LED22 Error9
LED25 Error8
LEFF8 IFIL
LE7F1 dwADD_Etc
LE7D1 dwAND_Etc
LE79D dwBIT_Etc
LE857 dwCALL_Etc
LE773 dwFixedByte
LE76E dwLDI_Etc
LE834 dwDEC_Etc
LE8DB dwDEFB
LEB6C dwDEFM

LE8FC dwDEFS
LE8ED dwDEFW
LE915 dwDISP
LE8D5 dwDJNZ
LE91C dwENT

LE9FF dwIM

LE96B dwIN_Etc

LF127 dwIY_AndEnableHiddenMsg

LE874 dwJP_Etc
LE8A0 dwJR_Etc
LEA1C dwLD

LE90B dwORG

LE9BD dwOUT_Etc

LE792 dwPOP
LE792 dwPUSH
LE782 dwRET_Etc
LE7B5 dwRL_Etc

LE777 dwRST

LE869 Plant_Word

LF13D EncryptedCopyrightMsg
LF652 InputChar
LF503 PrintChar
LE5A3 PrintDecimal
LE5A5 PD_HaveLZFlag
LE5C9 PD_Digit
LE5CA PD_Digit_Lp
LF2DF PrintHexByte
LE571 PrintHexWord
LE57F PrintHex8
LE596 PrintHex4
LE4E3 PrintStringZ
LE4E5 PrintStringZLp
LE4EF PrintStringZX

LE5D9 DeclareLabel
LE5E5 DL_GetLp
LE5F3 DL_GetDone
LE5F4 DL_FillSpcLp

LF6E2 InputLine
LFE00 Buffer
LF4CB PrinterFlag
LE59E PrintDecNZ

LE66A IsThisAFlag_Z
LE691 IsThisA16BitReg1_Z
LE6A2 IsThisA16BitReg2_Z
LE722 IsThisIY_Z
LE72A IsThisAn8BitReg_Z
LEC52 PlantByteA


LF20A MonModifySlash
LF24E MonModifyChar
LF214 MonModifyLp
LF234 MonModifyWriteByte
LF238 MonModifyNumError
LF23B MonModifyEOL

LF269 MonModifyAddrError
LF271 MonModifyNoParam
LF2F3 PrintHexNibble
LF344 MonCopyFail
LF375 MonSkipSpcEOL
LF378 MonSkipSpcOK

LF390 MinExecuteFail
LF3E0 MonTab_Next
LF40B MonNumericError
LFD0D Flag
LF51E PrintCharSaveReg
LF52A PrintCharInner
LF584 PC_Ctrl
LF5BB vPC_ShowCursor
LF4F4 PC_ShowCursor
LF53D PC_MoveRight
LF5C5 PC_MoveLeft
LF5E8 PC_MoveDown
LF5F4 PC_MoveUp
LF60A PC_Delete
LF5BE PC_Cret
LF6F7 PC_Insert
LF7B6 PC_ClearLine
LF728 PC_DeleteRight
LF618 PC_Tab
LF517 PC_NotCtrl
LF51A PC_NotCret
LF51C PC_NotPrinter
LF821 DrivePrinterA
LF63B PC_DrawChar
LF64B PC_DC_Lp
LF60F PC_D_Lp
LF624 PC_Tab_Lp
LF4F1 PC_StoreCurAndExit
LF70B PC_Insert_Lp
LF71F PC_InsertClr_Lp
LF71A PC_InsertClr
LF736 PC_DeleteLp
LF795 CompChar
LF799 CompCharLp
LF784 FindCharLp
LF77D FindChar
LF792 FC_Found
LF76E ReadLineLp
LF74E ReadLine
LF765 ReadLine1
LF5D1 PC_ML_AtSOL
LF5E2 PC_ML_Up
LF5CB PC_ML_X
LF54C PC_MR_Wrap
LF557 PC_MR_EOS
LF7C6 ScrollUp
LF7CB ScrollUpLp
LF7D8 ScrollUpInner
LF563 ClearBottomLine
LF566 ClearCharLine
LF575 ClearPixelLine
LF569 ClearCharLineLp
LF7A7 CompCharX
LF810 PrintLineFromDE
LF813 PrintLineLp

LF65F InputChar1
LF67D InputChar2
LF68C InputChar3
LF6A1 InputChar4
LF6AF InputChar5
LF6B8 InputChar6
LF6C8 InputChar7
LF6D5 InputChar8
LF6DF InputCharX
LF7AB Delay
LF7AD DelayLp
LF7AF DelayLp2

oe000
c                       emulate_spectrum "48K"
c                       zeusdatafollows PrintStringZ
_
c; Assembler stuff to start with, written almost entirely by Neil...
c; The source is long-gone and I've just guessed reasonable routine names
_
cZeus_PC                 equ *
_

i@E010
_
c                        ; Saved registers on stack, stored SP
_
c                        ; Setup attributes white/blue
i5
_
c                        ; Setup border colour
i5
_
c                        ; Gawd knows?
i5
_
c                        ; Disable printing
i4
_
c                        ; And something else
i3
_
c                        ; Gawd knows
i7
_
c                        ; Announce us.

i@E030
b1
a15
b2
aT00
_
c                        ; NOPs here because some bastard clumsily removed our names :(
_
i@E069
_
i@E079
_
c                        ; Check if this is a restart
_
i@E086
c                        ; Start from cold
_
i@E092
_
c                        ; Warm start
_
i@E0AB
aT00
_
i@E0F3
aT00
_
i@E154
c                        ; Reset the hidden message for any command that doesn't show it
_
i@E218
aT00
i@E487
aT00
i@E492
aT00
i@E635
aT00
i@ED0A
c; Turn the flow warnings off, this horrible code plants "LD HL,NNNN" instructions to skip over the following "LD B,NN"
c; instructions. Typical bloody Neil, saving a handful of bytes.
_
czpush zoWarnFlow
czoWarnFlow=false
_
i2
[9
b1
i2
]
_
czoWarnFlow=zpop
_
i@ED2A
aT00
i@ED86
aT00
i@EDA2
aT00
i@EDC7
aT00
i@EDCB
[$13
a2
]
aT08
aT08
b1
LE0BA AssEntryLp
LE0E6 AssNotLetter
LE0F0 AssShowError
LE0FC AssDealWithCmd
LE161 AssHaveLineNumber
LE174 AHLN_NotSpace
LE190 AHLN_NotFound
LE1A9 AHLN_NotFirstLine
LE1B5 CopyLineInBuffToHL
LE1B6 CopyLineInBuffToDE
LE1C1 AssReplaceLine
LE1DB ARL_ReplaceShorter
LE1EC RemoveSpaceFromSource
LE1E7 ARL_ReplaceWithEmpty
LE206 InsertSpaceForLine
LE22C ISFL_IsRoom
LE244 TL_NextCharLp1
LE245 TL_NextCharLp
LE246 TokeniseLine
LE251 TL_NotSp
LE25E TL_NoSpaces
LE261 TL_HasSpace
LE272 TL_NotAToken
LE27E TL_NotTokenOrIdent
LE336 FindLineLengthBC
LE3DE CmdRenumberLp
LE3E3 CmdRenumber_FindEOL
LEB8C CmdAssemble
LE315 FindLineNum_DE
LE31D FLN_Lp
LE32F FLN_FindLineEnd
LE330 FLN_FLELp

LE3F4 CmdDelete
LE417 CmdFind
LE3B5 CmdInsert
LE3BC CmdList
LF1A2 CmdMonitor
LE44A CmdNew
LE44F CmdOld
LE451 SetNoSourceFlag
LF130 CmdPrinter
LED66 CmdQuit
LE3CB CmdRenumber
LE45D CmdSymbol
LED83 CmdShowStart
LF4AA CmdShowHidden
LE446 CmdExecute

c;Command table
c;A
b1
W2
w2
c;D
b1
W4
w2
c;F
b1
W6
w2
c;I
b1
W4
w2
c;L
b1
W6
w2
c;M
b1
w2
c;N
b1
W2
w2
c;O
b1
W2
w2
c;P
b1
W2
w2
c;Q
b1
w2
c;R
b1
W6
w2
c;S
b1
W2
w2
c;T
b1
w2
c;What (etc)
b1
w2
c;X
b1
w2


b@EE57
_
c; Token $80
aT0A
_
[2
aT08
_
]
[2
aT0A
_
]
aT08
_
aT0A
_
aT0A
_
c; Token $88
_
aT08
_
aT0A
_
aT08
_
aT0A
_
aT08
_
[3
aT0A
_
]
c; Token $90
_
[5
aT0A
_
]
[3
aT08
_
]
c; Token $98
_
[2
aT08
_
]
[1
aT0A
_
]
[2
aT08
_
]
[3
aT0A
_
]
c; Token $A0
_
[2
aT08
_
]
[5
aT0A
_
]
[1
aT08
_
]
c; Token $A8
_
[2
aT08
_
]

[6
aT0A
_
]
c; Token $B0
_
[2
aT08
_
]
[1
aT0A
_
]
[1
aT08
_
]
[4
aT0A
_
]
c; Token $B8
_
[6
aT0A
_
]
[2
aT08
_
]
c; Token $C0
_
[2
aT0A
_
]
[1
aT08
_
]
[5
aT0A
_
]
c; Token $C8
_
[2
aT08
_
]
[1
aT0A
_
]
[1
aT08
_
]
[3
aT0A
_
]
[1
aT08
_
]
c; Token $D0
_
[1
aT0A
_
]
[1
aT08
_
]
[2
aT0A
_
]
[1
aT08
_
]
[1
aT0A
_
]
[1
aT08
_
]
[1
aT0A
_
]
c; Token $D8
_
[1
aT0A
_
]
[2
aT08
_
]
[1
aT0A
_
]
[2
aT08
_
]
[1
aT0A
_
]
[1
aT08
_
]
c; Token $E0
_
[2
aT08
_
]
[1
aT0A
_
]
[1
aT08
_
]
aT0A
_
b1
_

a@EFF8
_
c; Token $80
[8
b1
w2
]
_
c; Token $88
[8
b1
w2
]
_
c; Token $90
[8
b1
w2
]
_
c; Token $98
[8
b1
w2
]
_
c; Token $A0
[8
b1
w2
]
_
c; Token $A8
[8
b1
w2
]
_
c; Token $B0
[8
b1
w2
]
_
c; Token $B8
[8
b1
w2
]
_
c; Token $C0
[8
b1
w2
]
_
c; Token $C8
[8
b1
w2
]
_
c; Token $D0
[8
b1
w2
]
_
c; Token $D8
[8
b1
w2
]
_
c; Token $E0
[5
b1
w2
]
_

i@F13D
c; Encoded "Copyright Neil..." (etc)

bT00
_
c; Monitor stuff below this, written by Crem...
c; Spot the much simpler style.
_
i@F1A5
aT00
i@F1FD
aT00
i@F245
aT00
i@F255
aT00
i@F279
aT00
i@F347
aT00
i@F393
aT00
i@F48A
aT00
i@F498
aT00
i@F4B8
c; Encoded "42 "+Cret
bT00
_
w2
w2
w2
w2
_
[6
b1
]
_

i@F838
_
b21
_
b21

oFD00
w2
w2
w2
b1
w2
w2
w2
b2
w2
w2
w2
w2
w2
w2
w2
w2

oFE00
b20
_

LF481 MonCmdA
LF1B3 MonitorLp
LF301 MonCopy
LF3B5 MonTabulate
LF3DD MonTab_Lp
LF3E4 MonTab_Lp1
LF3EF MonTab_Lp2
LF204 MonModify
LF37B MonExecute
LF43F MonOutput
LF1FA MonZeus
LF463 MonInput
LF1EB MonSetTabs
LF411 MonSetColours
LF439 MSC_DoIt
LF85A TabStopData
LEDCB AssCmdTable
LE2D4 Tokenize
LE2EA TokenSameStartChar
LE2EB TokenCmpLp
LE2DA Token1
LE306 Token2
LE311 Token3
LE300 Token4
LE303 Token5
LE603 DL_MoveToNextSym
LE606 DL_FindNextSymLp
LE60B DL_FNS_Next
LE613 DL_CompareLp
LE64F DL_EnoughRoom
LE660 DL_Done

LEE06 AddCmdOffsets
LE114 DriveTable
LE10F DT_Lp
LE145 DT_Fail
LE134 DT_GetNextParam

LEE57 TokenDefnTable
LFD19 AssParam1
LFD1B AssParam2
LFD1D AssParam3

LF4B8 sHiddenMsg2
LF4AD PrintHiddenHL
LF4AB HiddenMsgBase

LF4BE Cur_pScreen
LF4C0 UnkWordVar2
LF4C2 UnkWordVar3
LF4C4 UnkWordVar4
LF4C6 UnkByteVar1
LF4C7 NoSourceFlag
LF4C8 UnkByteVar3
LF4C9 UnkByteVar4
LF4CA UnkByteVar5

LF4CC CLS
LF4ED BdrCol
LF838 TabStopDefn
LF859 TabStopBuffer
LF252 AddrError
LF276 NumError
LF4DF CLS_Attrs
LF4E6 CLS_cAttr

LF369 MonSkipSpc
LF28C MonReadNumHL
LF2CA MonReadNumA

LFD00 SourceStartAddr
LFD02 SourceEndAddr
LFD04 MagicNumber
LFD06 LastCmdIndex
LFD0B LineLength
LFD0F AssPC
LFD11 AssDISP
LFD13 SymbolTableBase
LFD17 EntryStackBase
*/

